@@ -1,4 +1,11 @@
-/* $Id: fileio.cpp 16273 2009-05-10 21:33:55Z rubidium $ */
+/* $Id: fileio.cpp 19430 2010-03-15 22:52:39Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file fileio.cpp Standard In/Out file operations */
 
@@ -11,7 +18,13 @@
 #include "tar_type.h"
 #ifdef WIN32
 #include <windows.h>
+#elif defined(__HAIKU__)
+#include <Path.h>
+#include <storage/FindDirectory.h>
 #else
+#if defined(OPENBSD) || defined(DOS)
+#include <unistd.h>
+#endif
 #include <pwd.h>
 #endif
 #include <sys/stat.h>
@@ -205,7 +218,7 @@ void FioOpenFile(int slot, const char *filename)
 	FioSeekToFile(slot, pos);
 }
 
-const char *_subdirs[NUM_SUBDIRS] = {
+static const char * const _subdirs[NUM_SUBDIRS] = {
 	"",
 	"save" PATHSEP,
 	"save" PATHSEP "autosave" PATHSEP,
@@ -265,6 +278,13 @@ char *FioFindFullPath(char *buf, size_t buflen, Subdirectory subdir, const char
 	FOR_ALL_SEARCHPATHS(sp) {
 		FioGetFullPath(buf, buflen, sp, subdir, filename);
 		if (FileExists(buf)) break;
+#if !defined(WIN32)
+		/* Be, as opening files, aware that sometimes the filename
+		 * might be in uppercase when it is in lowercase on the
+		 * disk. Ofcourse Windows doesn't care about casing. */
+		strtolower(buf + strlen(_searchpaths[sp]) - 1);
+		if (FileExists(buf)) break;
+#endif
 	}
 
 	return buf;
@@ -295,7 +315,7 @@ char *FioGetDirectory(char *buf, size_t buflen, Subdirectory subdir)
 	return buf;
 }
 
-FILE *FioFOpenFileSp(const char *filename, const char *mode, Searchpath sp, Subdirectory subdir, size_t *filesize)
+static FILE *FioFOpenFileSp(const char *filename, const char *mode, Searchpath sp, Subdirectory subdir, size_t *filesize)
 {
 #if defined(WIN32) && defined(UNICODE)
 	/* fopen is implemented as a define with ellipses for
@@ -337,7 +357,7 @@ FILE *FioFOpenFileSp(const char *filename, const char *mode, Searchpath sp, Subd
 FILE *FioFOpenFileTar(TarFileListEntry *entry, size_t *filesize)
 {
 	FILE *f = fopen(entry->tar_filename, "rb");
-	assert(f != NULL);
+	if (f == NULL) return f;
 
 	fseek(f, entry->position, SEEK_SET);
 	if (filesize != NULL) *filesize = entry->size;
@@ -543,7 +563,11 @@ bool TarListAddFile(const char *filename)
 	if (it != _tar_list.end()) return false;
 
 	FILE *f = fopen(filename, "rb");
-	assert(f != NULL);
+	/* Although the file has been found there can be
+	 * a number of reasons we cannot open the file.
+	 * Most common case is when we simply have not
+	 * been given read access. */
+	if (f == NULL) return false;
 
 	const char *dupped_filename = strdup(filename);
 	_tar_list[filename].filename = dupped_filename;
@@ -723,6 +747,79 @@ bool TarListAddFile(const char *filename)
 	return true;
 }
 
+/**
+ * Extract the tar with the given filename in the directory
+ * where the tar resides.
+ * @param tar_filename the name of the tar to extract.
+ * @return false on failure.
+ */
+bool ExtractTar(const char *tar_filename)
+{
+	TarList::iterator it = _tar_list.find(tar_filename);
+	/* We don't know the file. */
+	if (it == _tar_list.end()) return false;
+
+	const char *dirname = (*it).second.dirname;
+
+	/* The file doesn't have a sub directory! */
+	if (dirname == NULL) return false;
+
+	char filename[MAX_PATH];
+	strecpy(filename, tar_filename, lastof(filename));
+	char *p = strrchr(filename, PATHSEPCHAR);
+	/* The file's path does not have a separator? */
+	if (p == NULL) return false;
+
+	p++;
+	strecpy(p, dirname, lastof(filename));
+	DEBUG(misc, 8, "Extracting %s to directory %s", tar_filename, filename);
+	FioCreateDirectory(filename);
+
+	for (TarFileList::iterator it2 = _tar_filelist.begin(); it2 != _tar_filelist.end(); it2++) {
+		if (strcmp((*it2).second.tar_filename, tar_filename) != 0) continue;
+
+		strecpy(p, (*it2).first.c_str(), lastof(filename));
+
+		DEBUG(misc, 9, "  extracting %s", filename);
+
+		/* First open the file in the .tar. */
+		size_t to_copy = 0;
+		FILE *in = FioFOpenFileTar(&(*it2).second, &to_copy);
+		if (in == NULL) {
+			DEBUG(misc, 6, "Extracting %s failed; could not open %s", filename, tar_filename);
+			return false;
+		}
+
+		/* Now open the 'output' file. */
+		FILE *out = fopen(filename, "wb");
+		if (out == NULL) {
+			DEBUG(misc, 6, "Extracting %s failed; could not open %s", filename, filename);
+			fclose(in);
+			return false;
+		}
+
+		/* Now read from the tar and write it into the file. */
+		char buffer[4096];
+		size_t read;
+		for (; to_copy != 0; to_copy -= read) {
+			read = fread(buffer, 1, min(to_copy, lengthof(buffer)), in);
+			if (read <= 0 || fwrite(buffer, 1, read, out) != read) break;
+		}
+
+		/* Close everything up. */
+		fclose(in);
+		fclose(out);
+
+		if (to_copy != 0) {
+			DEBUG(misc, 6, "Extracting %s failed; still %i bytes to copy", filename, (int)to_copy);
+			return false;
+		}
+	}
+
+	DEBUG(misc, 9, "  extraction successful");
+	return true;
+}
+
 static int ScanPathForTarFiles(const char *path, size_t basepath_length)
 {
 	extern bool FiosIsValidFile(const char *path, const struct dirent *ent, struct stat *sb);
@@ -807,7 +904,7 @@ void ChangeWorkingDirectory(const char *exe)
 
 	if (app_bundle != NULL) app_bundle[0] = '\0';
 #endif /* WITH_COCOA */
-	char *s = (char*)strrchr(exe, PATHSEPCHAR);
+	char *s = const_cast<char *>(strrchr(exe, PATHSEPCHAR));
 	if (s != NULL) {
 		*s = '\0';
 #if defined(__DJGPP__)
@@ -831,6 +928,11 @@ void DetermineBasePaths(const char *exe)
 	char tmp[MAX_PATH];
 #if defined(__MORPHOS__) || defined(__AMIGA__) || defined(DOS) || defined(OS2) || !defined(WITH_PERSONAL_DIR)
 	_searchpaths[SP_PERSONAL_DIR] = NULL;
+#else
+#ifdef __HAIKU__
+	BPath path;
+	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
+	const char *homedir = path.Path();
 #else
 	const char *homedir = getenv("HOME");
 
@@ -838,6 +940,7 @@ void DetermineBasePaths(const char *exe)
 		const struct passwd *pw = getpwuid(getuid());
 		homedir = (pw == NULL) ? "" : pw->pw_dir;
 	}
+#endif
 
 	snprintf(tmp, MAX_PATH, "%s" PATHSEP "%s", homedir, PERSONAL_DIR);
 	AppendPathSeparator(tmp, MAX_PATH);
@@ -862,11 +965,16 @@ void DetermineBasePaths(const char *exe)
 #endif
 
 	/* Change the working directory to that one of the executable */
-	ChangeWorkingDirectory((char*)exe);
+	ChangeWorkingDirectory(exe);
 	if (getcwd(tmp, MAX_PATH) == NULL) *tmp = '\0';
 	AppendPathSeparator(tmp, MAX_PATH);
 	_searchpaths[SP_BINARY_DIR] = strdup(tmp);
 
+	if (_searchpaths[SP_WORKING_DIR] != NULL) {
+		/* Go back to the current working directory. */
+		ChangeWorkingDirectory(_searchpaths[SP_WORKING_DIR]);
+	}
+
 #if defined(__MORPHOS__) || defined(__AMIGA__) || defined(DOS) || defined(OS2)
 	_searchpaths[SP_INSTALLATION_DIR] = NULL;
 #else
@@ -900,7 +1008,7 @@ void DeterminePaths(const char *exe)
 
 	if (_config_file != NULL) {
 		_personal_dir = strdup(_config_file);
-		char *end = strrchr(_personal_dir , PATHSEPCHAR);
+		char *end = strrchr(_personal_dir, PATHSEPCHAR);
 		if (end == NULL) {
 			_personal_dir[0] = '\0';
 		} else {
@@ -945,7 +1053,7 @@ void DeterminePaths(const char *exe)
 	};
 
 	for (uint i = 0; i < lengthof(default_subdirs); i++) {
-		char *dir = str_fmt("%s%s", _personal_dir, FioGetSubdirectory(default_subdirs[i]));
+		char *dir = str_fmt("%s%s", _personal_dir, _subdirs[default_subdirs[i]]);
 		FioCreateDirectory(dir);
 		free(dir);
 	}
@@ -956,9 +1064,9 @@ void DeterminePaths(const char *exe)
 	FioCreateDirectory(_searchpaths[SP_AUTODOWNLOAD_DIR]);
 
 	/* Create the directory for each of the types of content */
-	const Subdirectory dirs[] = { SCENARIO_DIR, HEIGHTMAP_DIR, DATA_DIR, AI_DIR, AI_LIBRARY_DIR };
+	const Subdirectory dirs[] = { SCENARIO_DIR, HEIGHTMAP_DIR, DATA_DIR, AI_DIR, AI_LIBRARY_DIR, GM_DIR };
 	for (uint i = 0; i < lengthof(dirs); i++) {
-		char *tmp = str_fmt("%s%s", _searchpaths[SP_AUTODOWNLOAD_DIR], FioGetSubdirectory(dirs[i]));
+		char *tmp = str_fmt("%s%s", _searchpaths[SP_AUTODOWNLOAD_DIR], _subdirs[dirs[i]]);
 		FioCreateDirectory(tmp);
 		free(tmp);
 	}
@@ -1021,9 +1129,11 @@ void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 /**
  * Scan a single directory (and recursively it's children) and add
  * any graphics sets that are found.
+ * @param fs              the file scanner to add the files to
  * @param extension       the extension of files to search for.
  * @param path            full path we're currently at
  * @param basepath_length from where in the path are we 'based' on the search path
+ * @param recursive       whether to recursively search the sub directories
  */
 static uint ScanPath(FileScanner *fs, const char *extension, const char *path, size_t basepath_length, bool recursive)
 {
@@ -1071,6 +1181,7 @@ static uint ScanPath(FileScanner *fs, const char *extension, const char *path, s
 
 /**
  * Scan the given tar and add graphics sets when it finds one.
+ * @param fs        the file scanner to scan for
  * @param extension the extension of files to search for.
  * @param tar       the tar to search in.
  */
@@ -1097,6 +1208,7 @@ static uint ScanTar(FileScanner *fs, const char *extension, TarFileList::iterato
  * @param extension the extension of files to search for.
  * @param sd        the sub directory to search in.
  * @param tars      whether to search in the tars too.
+ * @param recursive whether to search recursively
  * @return the number of found files, i.e. the number of times that
  *         AddFile returned true.
  */
@@ -1125,6 +1237,7 @@ uint FileScanner::Scan(const char *extension, Subdirectory sd, bool tars, bool r
  * Scan for files with the given extention in the given search path.
  * @param extension the extension of files to search for.
  * @param directory the sub directory to search in.
+ * @param recursive whether to search recursively
  * @return the number of found files, i.e. the number of times that
  *         AddFile returned true.
  */
