@@ -1,4 +1,11 @@
-/* $Id: gamelog.cpp 16273 2009-05-10 21:33:55Z rubidium $ */
+/* $Id: gamelog.cpp 18809 2010-01-15 16:41:15Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file gamelog.cpp Definition of functions used for logging of important changes in the game */
 
@@ -9,7 +16,6 @@
 #include "variables.h"
 #include "string_func.h"
 #include "settings_type.h"
-#include "gamelog.h"
 #include "gamelog_internal.h"
 #include "console_func.h"
 #include "debug.h"
@@ -100,6 +106,7 @@ static void AddDebugText(char *buf, const char *s, ...)
 
 
 /** Prints GRF filename if found
+ * @param buf The location in the _dbgofs buffer to draw
  * @param grfid GRF which filename to print
  */
 static void PrintGrfFilename(char *buf, uint grfid)
@@ -112,6 +119,7 @@ static void PrintGrfFilename(char *buf, uint grfid)
 }
 
 /** Prints GRF ID, checksum and filename if found
+ * @param buf The location in the _dbgofs buffer to draw
  * @param grfid GRF ID
  * @param md5sum array of md5sum to print
  */
@@ -130,7 +138,7 @@ static void PrintGrfInfo(char *buf, uint grfid, const uint8 *md5sum)
 
 
 /** Text messages for various logged actions */
-static const char *la_text[] = {
+static const char * const la_text[] = {
 	"new game started",
 	"game loaded",
 	"GRF config changed",
@@ -143,7 +151,10 @@ static const char *la_text[] = {
 assert_compile(lengthof(la_text) == GLAT_END);
 
 
-/** Prints active gamelog */
+/**
+ * Prints active gamelog
+ * @param proc the procedure to draw with
+ */
 void GamelogPrint(GamelogPrintProc *proc)
 {
 	char buf[GAMELOG_BUF_LEN];
@@ -327,8 +338,11 @@ static LoggedChange *GamelogChange(GamelogChangeType ct)
  */
 void GamelogEmergency()
 {
-	assert(_gamelog_action_type == GLAT_EMERGENCY);
+	/* Terminate any active action */
+	if (_gamelog_action_type != GLAT_NONE) GamelogStopAction();
+	GamelogStartAction(GLAT_EMERGENCY);
 	GamelogChange(GLCT_EMERGENCY);
+	GamelogStopAction();
 }
 
 /** Finds out if current game is a loaded emergency savegame.
@@ -349,7 +363,6 @@ bool GamelogTestEmergency()
 }
 
 /** Logs a change in game revision
- * @param revision new revision string
  */
 void GamelogRevision()
 {
@@ -471,7 +484,8 @@ static void GamelogGRFBug(uint32 grfid, byte bug, uint64 data)
  * Ensures this is logged only once for each GRF and engine type
  * This check takes some time, but it is called pretty seldom, so it
  * doesn't matter that much (ideally it shouldn't be called at all).
- * @param engine engine to log
+ * @param grfid the broken NewGRF
+ * @param internal_id the internal ID of whatever's broken in the NewGRF
  * @return true iff a unique record was done
  */
 bool GamelogGRFBugReverse(uint32 grfid, uint16 internal_id)
@@ -591,7 +605,7 @@ void GamelogGRFAddList(const GRFConfig *newg)
 /** List of GRFs using array of pointers instead of linked list */
 struct GRFList {
 	uint n;
-	const GRFConfig *grf[VARARRAY_SIZE];
+	const GRFConfig *grf[];
 };
 
 /** Generates GRFList
