@@ -1,11 +1,18 @@
-/* $Id: livery.h 11874 2008-01-16 11:25:15Z rubidium $ */
+/* $Id: livery.h 17248 2009-08-21 20:21:05Z rubidium $ */
 
-/** @file livery.h */
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/** @file livery.h Functions/types related to livery colours. */
 
 #ifndef LIVERY_H
 #define LIVERY_H
 
-#include "player_type.h"
+#include "company_type.h"
 
 /* List of different livery schemes. */
 enum LiveryScheme {
@@ -67,10 +74,10 @@ struct Livery {
 };
 
 /**
- * Reset the livery schemes to the player's primary colour.
- * This is used on loading games without livery information and on new player start up.
- * @param p Player to reset.
+ * Reset the livery schemes to the company's primary colour.
+ * This is used on loading games without livery information and on new company start up.
+ * @param c Company to reset.
  */
-void ResetPlayerLivery(Player *p);
+void ResetCompanyLivery(Company *c);
 
 #endif /* LIVERY_H */
