@@ -1,4 +1,11 @@
-/* $Id: qtmidi.cpp 15718 2009-03-15 00:32:18Z rubidium $ */
+/* $Id: qtmidi.cpp 17570 2009-09-19 09:51:14Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /**
  * @file qtmidi.cpp
@@ -19,36 +26,18 @@
  */
 
 
-#define MAC_OS_X_VERSION_MIN_REQUIRED    MAC_OS_X_VERSION_10_3
-#include <AvailabilityMacros.h>
-
-/*
- * OpenTTD includes.
- */
-#define  WindowClass OSX_WindowClass
-#include <QuickTime/QuickTime.h>
-#undef   WindowClass
+#ifndef NO_QUICKTIME
 
 #include "../stdafx.h"
 #include "qtmidi.h"
-
-/*
- * System includes. We need to workaround with some defines because there's
- * stuff already defined in QuickTime headers.
- */
-#define  OTTD_Random OSX_OTTD_Random
-#undef   OTTD_Random
-#undef   WindowClass
-#undef   SL_ERROR
-#undef   bool
-
-#include <assert.h>
-#include <unistd.h>
-#include <fcntl.h>
-
-/* we need to include debug.h after CoreServices because defining DEBUG will break CoreServices in OSX 10.2 */
 #include "../debug.h"
 
+#define Rect  OTTDRect
+#define Point OTTDPoint
+#include <QuickTime/QuickTime.h>
+#undef Rect
+#undef Point
+
 static FMusicDriver_QtMidi iFMusicDriver_QtMidi;
 
 
@@ -61,7 +50,7 @@ enum {
  * Sets the @c OSType of a given file to @c 'Midi', but only if it's not
  * already set.
  *
- * @param *spec A @c FSSpec structure referencing a file.
+ * @param *ref A @c FSSpec structure referencing a file.
  */
 static void SetMIDITypeIfNeeded(const FSRef *ref)
 {
@@ -121,8 +110,9 @@ static bool LoadMovieForMIDIFile(const char *path, Movie *moov)
 	if (ret < 4) return false;
 
 	DEBUG(driver, 3, "qtmidi: header is '%.4s'", magic);
-	if (magic[0] != 'M' || magic[1] != 'T' || magic[2] != 'h' || magic[3] != 'd')
+	if (magic[0] != 'M' || magic[1] != 'T' || magic[2] != 'h' || magic[3] != 'd') {
 		return false;
+	}
 
 	if (noErr != FSPathMakeRef((const UInt8 *) path, &fsref, NULL)) return false;
 	SetMIDITypeIfNeeded(&fsref);
@@ -218,13 +208,15 @@ bool MusicDriver_QtMidi::IsSongPlaying()
 		case QT_STATE_STOP:
 			/* Do nothing. */
 			break;
+
 		case QT_STATE_PLAY:
 			MoviesTask(_quicktime_movie, 0);
 			/* Check wether movie ended. */
 			if (IsMovieDone(_quicktime_movie) ||
 					(GetMovieTime(_quicktime_movie, NULL) >=
-					 GetMovieDuration(_quicktime_movie)))
+					 GetMovieDuration(_quicktime_movie))) {
 				_quicktime_state = QT_STATE_STOP;
+			}
 	}
 
 	return _quicktime_state == QT_STATE_PLAY;
@@ -247,8 +239,11 @@ void MusicDriver_QtMidi::Stop()
 			DEBUG(driver, 3, "qtmidi: stopping not needed, already idle");
 			/* Do nothing. */
 			break;
+
 		case QT_STATE_PLAY:
 			StopSong();
+			/* Fall-through */
+
 		case QT_STATE_STOP:
 			DisposeMovie(_quicktime_movie);
 	}
@@ -272,12 +267,14 @@ void MusicDriver_QtMidi::PlaySong(const char *filename)
 		case QT_STATE_PLAY:
 			StopSong();
 			DEBUG(driver, 3, "qtmidi: previous tune stopped");
-			/* XXX Fall-through -- no break needed. */
+			/* Fall-through -- no break needed. */
+
 		case QT_STATE_STOP:
 			DisposeMovie(_quicktime_movie);
 			DEBUG(driver, 3, "qtmidi: previous tune disposed");
 			_quicktime_state = QT_STATE_IDLE;
-			/* XXX Fall-through -- no break needed. */
+			/* Fall-through -- no break needed. */
+
 		case QT_STATE_IDLE:
 			LoadMovieForMIDIFile(filename, &_quicktime_movie);
 			SetMovieVolume(_quicktime_movie, VOLUME);
@@ -297,11 +294,13 @@ void MusicDriver_QtMidi::StopSong()
 
 	switch (_quicktime_state) {
 		case QT_STATE_IDLE:
-			/* XXX Fall-through -- no break needed. */
+			/* Fall-through -- no break needed. */
+
 		case QT_STATE_STOP:
 			DEBUG(driver, 3, "qtmidi: stop requested, but already idle");
 			/* Do nothing. */
 			break;
+
 		case QT_STATE_PLAY:
 			StopMovie(_quicktime_movie);
 			_quicktime_state = QT_STATE_STOP;
@@ -330,9 +329,11 @@ void MusicDriver_QtMidi::SetVolume(byte vol)
 		case QT_STATE_IDLE:
 			/* Do nothing. */
 			break;
+
 		case QT_STATE_PLAY:
 		case QT_STATE_STOP:
 			SetMovieVolume(_quicktime_movie, VOLUME);
 	}
 }
 
+#endif /* NO_QUICKTIME */
