@@ -1,4 +1,11 @@
-/* $Id: openttd.cpp 16310 2009-05-15 10:24:13Z rubidium $ */
+/* $Id: openttd.cpp 19430 2010-03-15 22:52:39Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file openttd.cpp Functions related to starting OpenTTD. */
 
@@ -8,7 +15,6 @@
 #include "variables.h"
 #undef VARDEF
 
-#include "openttd.h"
 
 #include "blitter/factory.hpp"
 #include "sound/sound_driver.hpp"
@@ -16,20 +22,20 @@
 #include "video/video_driver.hpp"
 
 #include "fontcache.h"
-#include "gfxinit.h"
 #include "gui.h"
-#include "mixer.h"
 #include "sound_func.h"
 #include "window_func.h"
 
+#include "base_media_base.h"
 #include "saveload/saveload.h"
 #include "landscape.h"
 #include "company_func.h"
 #include "command_func.h"
 #include "news_func.h"
-#include "fileio_func.h"
 #include "fios.h"
 #include "aircraft.h"
+#include "roadveh.h"
+#include "train.h"
 #include "console_func.h"
 #include "screenshot.h"
 #include "network/network.h"
@@ -46,11 +52,18 @@
 #include "gamelog.h"
 #include "cheat_type.h"
 #include "animated_tile_func.h"
+#include "roadstop_base.h"
 #include "functions.h"
 #include "elrail_func.h"
 #include "rev.h"
 #include "highscore.h"
-#include "thread.h"
+#include "thread/thread.h"
+#include "station_base.h"
+#include "crashlog.h"
+#include "company_base.h"
+#include "engine_base.h"
+#include "engine_func.h"
+#include "core/random_func.hpp"
 
 #include "newgrf_commons.h"
 
@@ -72,7 +85,7 @@ void ProcessAsyncSaveFinish();
 void CallWindowTickEvent();
 
 extern void SetDifficultyLevel(int mode, DifficultySettings *gm_opt);
-extern Company *DoStartupNewCompany(bool is_ai);
+extern Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY);
 extern void ShowOSErrorBox(const char *buf, bool system);
 extern void InitializeRailGUI();
 
@@ -111,10 +124,10 @@ void CDECL error(const char *s, ...)
 	va_end(va);
 
 	ShowOSErrorBox(buf, true);
-	if (_video_driver != NULL) _video_driver->Stop();
 
-	assert(0);
-	exit(1);
+	/* Set the error message for the crash log and then invoke it. */
+	CrashLog::SetErrorMessage(buf);
+	abort();
 }
 
 /**
@@ -148,7 +161,6 @@ static void ShowHelp()
 		"  -s drv              = Set sound driver (see below) (param bufsize,hz)\n"
 		"  -m drv              = Set music driver (see below)\n"
 		"  -b drv              = Set the blitter to use (see below)\n"
-		"  -a ai               = Force use of specific AI (see below)\n"
 		"  -r res              = Set resolution (for instance 800x600)\n"
 		"  -h                  = Display this help text\n"
 		"  -t year             = Set starting year\n"
@@ -158,6 +170,8 @@ static void ShowHelp()
 		"  -G seed             = Set random seed\n"
 #if defined(ENABLE_NETWORK)
 		"  -n [ip:port#company]= Start networkgame\n"
+		"  -p password         = Password to join server\n"
+		"  -P password         = Password to join company\n"
 		"  -D [ip][:port]      = Start dedicated server\n"
 		"  -l ip[:port]        = Redirect DEBUG()\n"
 #if !defined(__MORPHOS__) && !defined(__AMIGA__) && !defined(WIN32)
@@ -169,6 +183,8 @@ static void ShowHelp()
 		"                        Default value (2) lets OpenTTD use the palette\n"
 		"                          specified in graphics set file (see below)\n"
 		"  -I graphics_set     = Force the graphics set (see below)\n"
+		"  -S sounds_set       = Force the sounds set (see below)\n"
+		"  -M music_set        = Force the music set (see below)\n"
 		"  -c config_file      = Use 'config_file' instead of 'openttd.cfg'\n"
 		"  -x                  = Do not automatically save to config file on exit\n"
 		"\n",
@@ -176,7 +192,13 @@ static void ShowHelp()
 	);
 
 	/* List the graphics packs */
-	p = GetGraphicsSetsList(p, lastof(buf));
+	p = BaseGraphics::GetSetsList(p, lastof(buf));
+
+	/* List the sounds packs */
+	p = BaseSounds::GetSetsList(p, lastof(buf));
+
+	/* List the music packs */
+	p = BaseMusic::GetSetsList(p, lastof(buf));
 
 	/* List the drivers */
 	p = VideoDriverFactoryBase::GetDriversInfo(p, lastof(buf));
@@ -204,7 +226,7 @@ struct MyGetOptData {
 	int numleft;
 	char **argv;
 	const char *options;
-	const char *cont;
+	char *cont;
 
 	MyGetOptData(int argc, char **argv, const char *options)
 	{
@@ -218,9 +240,7 @@ struct MyGetOptData {
 
 static int MyGetOpt(MyGetOptData *md)
 {
-	const char *s, *r, *t;
-
-	s = md->cont;
+	char *s = md->cont;
 	if (s != NULL)
 		goto md_continue_here;
 
@@ -232,12 +252,14 @@ static int MyGetOpt(MyGetOptData *md)
 md_continue_here:;
 			s++;
 			if (*s != 0) {
+				const char *r;
 				/* Found argument, try to locate it in options. */
 				if (*s == ':' || (r = strchr(md->options, *s)) == NULL) {
 					/* ERROR! */
 					return -2;
 				}
 				if (r[1] == ':') {
+					char *t;
 					/* Item wants an argument. Check if the argument follows, or if it comes as a separate arg. */
 					if (!*(t = s + 1)) {
 						/* It comes as a separate arg. Check if out of args? */
@@ -251,7 +273,7 @@ md_continue_here:;
 							md->argv++;
 						}
 					}
-					md->opt = (char*)t;
+					md->opt = t;
 					md->cont = NULL;
 					return *s;
 				}
@@ -291,7 +313,6 @@ static void InitializeDynamicVariables()
 	_house_mngr.ResetMapping();
 	_industry_mngr.ResetMapping();
 	_industile_mngr.ResetMapping();
-	_Company_pool.AddBlockToPool();
 }
 
 
@@ -316,16 +337,17 @@ static void ShutdownGame()
 
 	/* Uninitialize variables that are allocated dynamically */
 	GamelogReset();
-	_Town_pool.CleanPool();
-	_Industry_pool.CleanPool();
-	_Station_pool.CleanPool();
-	_Vehicle_pool.CleanPool();
-	_Sign_pool.CleanPool();
-	_Order_pool.CleanPool();
-	_Group_pool.CleanPool();
-	_CargoPacket_pool.CleanPool();
-	_Engine_pool.CleanPool();
-	_Company_pool.CleanPool();
+	_town_pool.CleanPool();
+	_industry_pool.CleanPool();
+	_station_pool.CleanPool();
+	_roadstop_pool.CleanPool();
+	_vehicle_pool.CleanPool();
+	_sign_pool.CleanPool();
+	_order_pool.CleanPool();
+	_group_pool.CleanPool();
+	_cargopacket_pool.CleanPool();
+	_engine_pool.CleanPool();
+	_company_pool.CleanPool();
 
 	free(_config_file);
 
@@ -345,14 +367,14 @@ static void LoadIntroGame()
 
 	/* Load the default opening screen savegame */
 	if (SaveOrLoad("opntitle.dat", SL_LOAD, DATA_DIR) != SL_OK) {
-		GenerateWorld(GW_EMPTY, 64, 64); // if failed loading, make empty world.
+		GenerateWorld(GWM_EMPTY, 64, 64); // if failed loading, make empty world.
 		WaitTillGeneratedWorld();
 		SetLocalCompany(COMPANY_SPECTATOR);
 	} else {
 		SetLocalCompany(COMPANY_FIRST);
 	}
 
-	_pause_game = 0;
+	_pause_mode = PM_UNPAUSED;
 	_cursor.fix_at = false;
 
 	CheckForMissingGlyphsInLoadedLanguagePack();
@@ -363,20 +385,24 @@ static void LoadIntroGame()
 
 void MakeNewgameSettingsLive()
 {
+#ifdef ENABLE_AI
 	for (CompanyID c = COMPANY_FIRST; c < MAX_COMPANIES; c++) {
 		if (_settings_game.ai_config[c] != NULL) {
 			delete _settings_game.ai_config[c];
 		}
 	}
+#endif /* ENABLE_AI */
 
 	_settings_game = _settings_newgame;
 
+#ifdef ENABLE_AI
 	for (CompanyID c = COMPANY_FIRST; c < MAX_COMPANIES; c++) {
 		_settings_game.ai_config[c] = NULL;
 		if (_settings_newgame.ai_config[c] != NULL) {
 			_settings_game.ai_config[c] = new AIConfig(_settings_newgame.ai_config[c]);
 		}
 	}
+#endif /* ENABLE_AI */
 }
 
 byte _savegame_sort_order;
@@ -393,6 +419,8 @@ int ttd_main(int argc, char *argv[])
 	char *videodriver = NULL;
 	char *blitter = NULL;
 	char *graphics_set = NULL;
+	char *sounds_set = NULL;
+	char *music_set = NULL;
 	Dimension resolution = {0, 0};
 	Year startyear = INVALID_YEAR;
 	uint generation_seed = GENERATE_NEW_SEED;
@@ -404,6 +432,8 @@ int ttd_main(int argc, char *argv[])
 	char *debuglog_conn = NULL;
 	char *dedicated_host = NULL;
 	uint16 dedicated_port = 0;
+	char *join_server_password = NULL;
+	char *join_company_password = NULL;
 #endif /* ENABLE_NETWORK */
 
 	_game_mode = GM_MENU;
@@ -416,7 +446,7 @@ int ttd_main(int argc, char *argv[])
 	 *   a letter means: it accepts that param (e.g.: -h)
 	 *   a ':' behind it means: it need a param (e.g.: -m<driver>)
 	 *   a '::' behind it means: it can optional have a param (e.g.: -d<debug>) */
-	optformat = "m:s:v:b:hD::n::ei::I:t:d::r:g::G:c:xl:"
+	optformat = "m:s:v:b:hD::n::ei::I:S:M:t:d::r:g::G:c:xl:p:P:"
 #if !defined(__MORPHOS__) && !defined(__AMIGA__) && !defined(WIN32)
 		"f"
 #endif
@@ -427,6 +457,8 @@ int ttd_main(int argc, char *argv[])
 	while ((i = MyGetOpt(&mgo)) != -1) {
 		switch (i) {
 		case 'I': free(graphics_set); graphics_set = strdup(mgo.opt); break;
+		case 'S': free(sounds_set); sounds_set = strdup(mgo.opt); break;
+		case 'M': free(music_set); music_set = strdup(mgo.opt); break;
 		case 'm': free(musicdriver); musicdriver = strdup(mgo.opt); break;
 		case 's': free(sounddriver); sounddriver = strdup(mgo.opt); break;
 		case 'v': free(videodriver); videodriver = strdup(mgo.opt); break;
@@ -461,6 +493,12 @@ int ttd_main(int argc, char *argv[])
 		case 'l':
 			debuglog_conn = mgo.opt;
 			break;
+		case 'p':
+			join_server_password = mgo.opt;
+			break;
+		case 'P':
+			join_company_password = mgo.opt;
+			break;
 #endif /* ENABLE_NETWORK */
 		case 'r': ParseResolution(&resolution, mgo.opt); break;
 		case 't': startyear = atoi(mgo.opt); break;
@@ -509,7 +547,9 @@ int ttd_main(int argc, char *argv[])
 			 * We can't do them earlier because then we can't show it on
 			 * the debug console as that hasn't been configured yet. */
 			DeterminePaths(argv[0]);
-			FindGraphicsSets();
+			BaseGraphics::FindSets();
+			BaseSounds::FindSets();
+			BaseMusic::FindSets();
 			ShowHelp();
 			return 0;
 		}
@@ -521,7 +561,9 @@ int ttd_main(int argc, char *argv[])
 #endif
 
 	DeterminePaths(argv[0]);
-	FindGraphicsSets();
+	BaseGraphics::FindSets();
+	BaseSounds::FindSets();
+	BaseMusic::FindSets();
 
 #if defined(UNIX) && !defined(__MORPHOS__)
 	/* We must fork here, or we'll end up without some resources we need (like sockets) */
@@ -539,13 +581,19 @@ int ttd_main(int argc, char *argv[])
 	if (startyear != INVALID_YEAR) _settings_newgame.game_creation.starting_year = startyear;
 	if (generation_seed != GENERATE_NEW_SEED) _settings_newgame.game_creation.generation_seed = generation_seed;
 
-	/* The width and height must be at least 1 pixel, this
-	 * way all internal drawing routines work correctly. */
-	if (_cur_resolution.width  <= 0) _cur_resolution.width  = 1;
-	if (_cur_resolution.height <= 0) _cur_resolution.height = 1;
+	/*
+	 * The width and height must be at least 1 pixel and width times
+	 * height must still fit within a 32 bits integer, this way all
+	 * internal drawing routines work correctly.
+	 */
+	_cur_resolution.width  = ClampU(_cur_resolution.width,  1, UINT16_MAX);
+	_cur_resolution.height = ClampU(_cur_resolution.height, 1, UINT16_MAX);
 
 #if defined(ENABLE_NETWORK)
-	if (dedicated_host) snprintf(_settings_client.network.server_bind_ip, sizeof(_settings_client.network.server_bind_ip), "%s", dedicated_host);
+	if (dedicated_host) {
+		_network_bind_list.Clear();
+		*_network_bind_list.Append() = strdup(dedicated_host);
+	}
 	if (dedicated_port) _settings_client.network.server_port = dedicated_port;
 	if (_dedicated_forks && !dedicated) _dedicated_forks = false;
 #endif /* ENABLE_NETWORK */
@@ -562,59 +610,78 @@ int ttd_main(int argc, char *argv[])
 	/* initialize all variables that are allocated dynamically */
 	InitializeDynamicVariables();
 
-	/* Sample catalogue */
-	DEBUG(misc, 1, "Loading sound effects...");
-	MxInitialize(11025);
-	SoundInitialize("sample.cat");
-
 	/* Initialize FreeType */
 	InitFreeType();
 
-	/* This must be done early, since functions use the InvalidateWindow* calls */
+	/* This must be done early, since functions use the SetWindowDirty* calls */
 	InitWindowSystem();
 
-	if (graphics_set == NULL) graphics_set = _ini_graphics_set;
-	if (!SetGraphicsSet(graphics_set)) {
+	/* Look for the sounds before the graphics. Otherwise none would be set and
+	 * the first initialisation of the video happens on the wrong data. Now it
+	 * can do the first initialisation right. */
+	if (sounds_set == NULL && BaseSounds::ini_set != NULL) sounds_set = strdup(BaseSounds::ini_set);
+	if (!BaseSounds::SetSet(sounds_set)) {
+		StrEmpty(sounds_set) ?
+			usererror("Failed to find a sounds set. Please acquire a sounds set for OpenTTD. See section 4.1 of readme.txt.") :
+			usererror("Failed to select requested sounds set '%s'", sounds_set);
+	}
+	free(sounds_set);
+
+	if (graphics_set == NULL && BaseGraphics::ini_set != NULL) graphics_set = strdup(BaseGraphics::ini_set);
+	if (!BaseGraphics::SetSet(graphics_set)) {
 		StrEmpty(graphics_set) ?
-			usererror("Failed to find a graphics set. Please acquire a graphics set for OpenTTD.") :
+			usererror("Failed to find a graphics set. Please acquire a graphics set for OpenTTD. See section 4.1 of readme.txt.") :
 			usererror("Failed to select requested graphics set '%s'", graphics_set);
 	}
+	free(graphics_set);
+
+	if (music_set == NULL && BaseMusic::ini_set != NULL) music_set = strdup(BaseMusic::ini_set);
+	if (!BaseMusic::SetSet(music_set)) {
+		StrEmpty(music_set) ?
+			usererror("Failed to find a music set. Please acquire a music set for OpenTTD. See section 4.1 of readme.txt.") :
+			usererror("Failed to select requested music set '%s'", music_set);
+	}
+	free(music_set);
 
 	/* Initialize game palette */
 	GfxInitPalettes();
 
 	DEBUG(misc, 1, "Loading blitter...");
-	if (blitter == NULL) blitter = _ini_blitter;
+	if (blitter == NULL && _ini_blitter != NULL) blitter = strdup(_ini_blitter);
 	if (BlitterFactoryBase::SelectBlitter(blitter) == NULL)
 		StrEmpty(blitter) ?
 			usererror("Failed to autoprobe blitter") :
 			usererror("Failed to select requested blitter '%s'; does it exist?", blitter);
+	free(blitter);
 
 	DEBUG(driver, 1, "Loading drivers...");
 
-	if (sounddriver == NULL) sounddriver = _ini_sounddriver;
+	if (sounddriver == NULL && _ini_sounddriver != NULL) sounddriver = strdup(_ini_sounddriver);
 	_sound_driver = (SoundDriver*)SoundDriverFactoryBase::SelectDriver(sounddriver, Driver::DT_SOUND);
 	if (_sound_driver == NULL) {
 		StrEmpty(sounddriver) ?
 			usererror("Failed to autoprobe sound driver") :
 			usererror("Failed to select requested sound driver '%s'", sounddriver);
 	}
+	free(sounddriver);
 
-	if (musicdriver == NULL) musicdriver = _ini_musicdriver;
-	_music_driver = (MusicDriver*)MusicDriverFactoryBase::SelectDriver(musicdriver, Driver::DT_MUSIC);
-	if (_music_driver == NULL) {
-		StrEmpty(musicdriver) ?
-			usererror("Failed to autoprobe music driver") :
-			usererror("Failed to select requested music driver '%s'", musicdriver);
-	}
-
-	if (videodriver == NULL) videodriver = _ini_videodriver;
+	if (videodriver == NULL && _ini_videodriver != NULL) videodriver = strdup(_ini_videodriver);
 	_video_driver = (VideoDriver*)VideoDriverFactoryBase::SelectDriver(videodriver, Driver::DT_VIDEO);
 	if (_video_driver == NULL) {
 		StrEmpty(videodriver) ?
 			usererror("Failed to autoprobe video driver") :
 			usererror("Failed to select requested video driver '%s'", videodriver);
 	}
+	free(videodriver);
+
+	if (musicdriver == NULL && _ini_musicdriver != NULL) musicdriver = strdup(_ini_musicdriver);
+	_music_driver = (MusicDriver*)MusicDriverFactoryBase::SelectDriver(musicdriver, Driver::DT_MUSIC);
+	if (_music_driver == NULL) {
+		StrEmpty(musicdriver) ?
+			usererror("Failed to autoprobe music driver") :
+			usererror("Failed to select requested music driver '%s'", musicdriver);
+	}
+	free(musicdriver);
 
 	_savegame_sort_order = SORT_BY_DATE | SORT_DESCENDING;
 	/* Initialize the zoom level of the screen to normal */
@@ -657,7 +724,7 @@ int ttd_main(int argc, char *argv[])
 	_genworld_paint_mutex->BeginCritical();
 	_genworld_mapgen_mutex->BeginCritical();
 
-	GenerateWorld(GW_EMPTY, 64, 64); // Make the viewport initialization happy
+	GenerateWorld(GWM_EMPTY, 64, 64); // Make the viewport initialization happy
 	WaitTillGeneratedWorld();
 
 	CheckForMissingGlyphsInLoadedLanguagePack();
@@ -667,26 +734,24 @@ int ttd_main(int argc, char *argv[])
 		if (network_conn != NULL) {
 			const char *port = NULL;
 			const char *company = NULL;
-			uint16 rport;
-
-			rport = NETWORK_DEFAULT_PORT;
-			_network_playas = COMPANY_NEW_COMPANY;
+			uint16 rport = NETWORK_DEFAULT_PORT;
+			CompanyID join_as = COMPANY_NEW_COMPANY;
 
 			ParseConnectionString(&company, &port, network_conn);
 
 			if (company != NULL) {
-				_network_playas = (CompanyID)atoi(company);
+				join_as = (CompanyID)atoi(company);
 
-				if (_network_playas != COMPANY_SPECTATOR) {
-					_network_playas--;
-					if (_network_playas >= MAX_COMPANIES) return false;
+				if (join_as != COMPANY_SPECTATOR) {
+					join_as--;
+					if (join_as >= MAX_COMPANIES) return false;
 				}
 			}
 			if (port != NULL) rport = atoi(port);
 
 			LoadIntroGame();
 			_switch_mode = SM_NONE;
-			NetworkClientConnectGame(NetworkAddress(network_conn, rport));
+			NetworkClientConnectGame(NetworkAddress(network_conn, rport), join_as, join_server_password, join_company_password);
 		}
 	}
 #endif /* ENABLE_NETWORK */
@@ -704,6 +769,14 @@ int ttd_main(int argc, char *argv[])
 	/* Reset windowing system, stop drivers, free used memory, ... */
 	ShutdownGame();
 
+	free(const_cast<char *>(BaseGraphics::ini_set));
+	free(const_cast<char *>(BaseSounds::ini_set));
+	free(const_cast<char *>(BaseMusic::ini_set));
+	free(_ini_musicdriver);
+	free(_ini_sounddriver);
+	free(_ini_videodriver);
+	free(_ini_blitter);
+
 	return 0;
 }
 
@@ -719,18 +792,6 @@ void HandleExitGameRequest()
 	}
 }
 
-static void ShowScreenshotResult(bool b)
-{
-	if (b) {
-		extern char _screenshot_name[];
-		SetDParamStr(0, _screenshot_name);
-		ShowErrorMessage(INVALID_STRING_ID, STR_031B_SCREENSHOT_SUCCESSFULLY, 0, 0);
-	} else {
-		ShowErrorMessage(INVALID_STRING_ID, STR_031C_SCREENSHOT_FAILED, 0, 0);
-	}
-
-}
-
 static void MakeNewGameDone()
 {
 	SettingsDisableElrail(_settings_game.vehicle.disable_elrails);
@@ -745,16 +806,13 @@ static void MakeNewGameDone()
 	/* Create a single company */
 	DoStartupNewCompany(false);
 
-	Company *c = GetCompany(COMPANY_FIRST);
-	c->engine_renew = _settings_client.gui.autorenew;
-	c->engine_renew_months = _settings_client.gui.autorenew_months;
-	c->engine_renew_money = _settings_client.gui.autorenew_money;
+	Company *c = Company::Get(COMPANY_FIRST);
+	c->settings = _settings_client.company;
 
 	IConsoleCmdExec("exec scripts/game_start.scr 0");
 
 	SetLocalCompany(COMPANY_FIRST);
 	_current_company = _local_company;
-	DoCommandP(0, (_settings_client.gui.autorenew << 15 ) | (_settings_client.gui.autorenew_months << 16) | 4, _settings_client.gui.autorenew_money, CMD_SET_AUTOREPLACE);
 
 	InitializeRailGUI();
 
@@ -762,26 +820,22 @@ static void MakeNewGameDone()
 	/* We are the server, we start a new company (not dedicated),
 	 * so set the default password *if* needed. */
 	if (_network_server && !StrEmpty(_settings_client.network.default_company_pass)) {
-		char *password = _settings_client.network.default_company_pass;
-		NetworkChangeCompanyPassword(1, &password);
+		NetworkChangeCompanyPassword(_settings_client.network.default_company_pass);
 	}
 #endif /* ENABLE_NETWORK */
 
 	MarkWholeScreenDirty();
 }
 
-static void MakeNewGame(bool from_heightmap)
+static void MakeNewGame(bool from_heightmap, bool reset_settings)
 {
 	_game_mode = GM_NORMAL;
 
 	ResetGRFConfig(true);
-	_engine_mngr.ResetToDefaultMapping();
-	_house_mngr.ResetMapping();
-	_industile_mngr.ResetMapping();
-	_industry_mngr.ResetMapping();
+	InitializeDynamicVariables();
 
 	GenerateWorldSetCallback(&MakeNewGameDone);
-	GenerateWorld(from_heightmap ? GW_HEIGHTMAP : GW_NEWGAME, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
+	GenerateWorld(from_heightmap ? GWM_HEIGHTMAP : GWM_NEWGAME, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y, reset_settings);
 }
 
 static void MakeNewEditorWorldDone()
@@ -796,7 +850,7 @@ static void MakeNewEditorWorld()
 	ResetGRFConfig(true);
 
 	GenerateWorldSetCallback(&MakeNewEditorWorldDone);
-	GenerateWorld(GW_EMPTY, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
+	GenerateWorld(GWM_EMPTY, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
 }
 
 void StartupCompanies();
@@ -816,7 +870,7 @@ static void StartScenario()
 	if (_file_to_saveload.mode == SL_INVALID) {
 		DEBUG(sl, 0, "Savegame is obsolete or invalid format: '%s'", _file_to_saveload.name);
 		SetDParamStr(0, GetSaveLoadErrorString());
-		ShowErrorMessage(INVALID_STRING_ID, STR_JUST_RAW_STRING, 0, 0);
+		ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
 		_game_mode = GM_MENU;
 		return;
 	}
@@ -832,7 +886,7 @@ static void StartScenario()
 	if (SaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, SCENARIO_DIR) != SL_OK) {
 		LoadIntroGame();
 		SetDParamStr(0, GetSaveLoadErrorString());
-		ShowErrorMessage(INVALID_STRING_ID, STR_JUST_RAW_STRING, 0, 0);
+		ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
 	}
 
 	_settings_game.difficulty = _settings_newgame.difficulty;
@@ -845,7 +899,8 @@ static void StartScenario()
 
 	SetLocalCompany(COMPANY_FIRST);
 	_current_company = _local_company;
-	DoCommandP(0, (_settings_client.gui.autorenew << 15 ) | (_settings_client.gui.autorenew_months << 16) | 4, _settings_client.gui.autorenew_money, CMD_SET_AUTOREPLACE);
+	Company *c = Company::Get(COMPANY_FIRST);
+	c->settings = _settings_client.company;
 
 	MarkWholeScreenDirty();
 }
@@ -888,7 +943,7 @@ void SwitchToMode(SwitchMode new_mode)
 	if (new_mode != SM_SAVE) {
 		/* If the network is active, make it not-active */
 		if (_networking) {
-			if (_network_server && (new_mode == SM_LOAD || new_mode == SM_NEWGAME)) {
+			if (_network_server && (new_mode == SM_LOAD || new_mode == SM_NEWGAME || new_mode == SM_RESTARTGAME)) {
 				NetworkReboot();
 			} else {
 				NetworkDisconnect();
@@ -921,13 +976,14 @@ void SwitchToMode(SwitchMode new_mode)
 			MakeNewEditorWorld();
 			break;
 
+		case SM_RESTARTGAME: // Restart --> 'Random game' with current settings
 		case SM_NEWGAME: // New Game --> 'Random game'
 #ifdef ENABLE_NETWORK
 			if (_network_server) {
 				snprintf(_network_game_info.map_name, lengthof(_network_game_info.map_name), "Random Map");
 			}
 #endif /* ENABLE_NETWORK */
-			MakeNewGame(false);
+			MakeNewGame(false, new_mode == SM_NEWGAME);
 			break;
 
 		case SM_START_SCENARIO: // New Game --> Choose one of the preset scenarios
@@ -946,7 +1002,7 @@ void SwitchToMode(SwitchMode new_mode)
 			if (!SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_NORMAL, NO_DIRECTORY)) {
 				LoadIntroGame();
 				SetDParamStr(0, GetSaveLoadErrorString());
-				ShowErrorMessage(INVALID_STRING_ID, STR_JUST_RAW_STRING, 0, 0);
+				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
 			} else {
 				if (_saveload_mode == SLD_LOAD_SCENARIO) {
 					StartupEngines();
@@ -957,7 +1013,7 @@ void SwitchToMode(SwitchMode new_mode)
 				/* Execute the game-start script */
 				IConsoleCmdExec("exec scripts/game_start.scr 0");
 				/* Decrease pause counter (was increased from opening load dialog) */
-				DoCommandP(0, 0, 0, CMD_PAUSE);
+				DoCommandP(0, PM_PAUSED_SAVELOAD, 0, CMD_PAUSE);
 #ifdef ENABLE_NETWORK
 				if (_network_server) {
 					snprintf(_network_game_info.map_name, lengthof(_network_game_info.map_name), "%s (Loaded game)", _file_to_saveload.title);
@@ -973,13 +1029,13 @@ void SwitchToMode(SwitchMode new_mode)
 				snprintf(_network_game_info.map_name, lengthof(_network_game_info.map_name), "%s (Heightmap)", _file_to_saveload.title);
 			}
 #endif /* ENABLE_NETWORK */
-			MakeNewGame(true);
+			MakeNewGame(true, true);
 			break;
 
 		case SM_LOAD_HEIGHTMAP: // Load heightmap from scenario editor
 			SetLocalCompany(OWNER_NONE);
 
-			GenerateWorld(GW_HEIGHTMAP, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
+			GenerateWorld(GWM_HEIGHTMAP, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
 			MarkWholeScreenDirty();
 			break;
 
@@ -987,32 +1043,36 @@ void SwitchToMode(SwitchMode new_mode)
 			if (SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_EDITOR, NO_DIRECTORY)) {
 				SetLocalCompany(OWNER_NONE);
 				_settings_newgame.game_creation.starting_year = _cur_year;
+				/* Cancel the saveload pausing */
+				DoCommandP(0, PM_PAUSED_SAVELOAD, 0, CMD_PAUSE);
 			} else {
 				SetDParamStr(0, GetSaveLoadErrorString());
-				ShowErrorMessage(INVALID_STRING_ID, STR_JUST_RAW_STRING, 0, 0);
+				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
 			}
 			break;
 		}
 
 		case SM_MENU: // Switch to game intro menu
 			LoadIntroGame();
+			if (BaseSounds::ini_set == NULL && BaseSounds::GetUsedSet()->fallback) {
+				ShowErrorMessage(STR_WARNING_FALLBACK_SOUNDSET, INVALID_STRING_ID, 0, 0, true);
+				BaseSounds::ini_set = strdup(BaseSounds::GetUsedSet()->name);
+			}
 			break;
 
 		case SM_SAVE: // Save game
 			/* Make network saved games on pause compatible to singleplayer */
-			if (_networking && _pause_game == 1) _pause_game = 2;
 			if (SaveOrLoad(_file_to_saveload.name, SL_SAVE, NO_DIRECTORY) != SL_OK) {
 				SetDParamStr(0, GetSaveLoadErrorString());
-				ShowErrorMessage(INVALID_STRING_ID, STR_JUST_RAW_STRING, 0, 0);
+				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, 0, 0);
 			} else {
 				DeleteWindowById(WC_SAVELOAD, 0);
 			}
-			if (_networking && _pause_game == 2) _pause_game = 1;
 			break;
 
 		case SM_GENRANDLAND: // Generate random land within scenario editor
 			SetLocalCompany(OWNER_NONE);
-			GenerateWorld(GW_RANDOM, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
+			GenerateWorld(GWM_RANDOM, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
 			/* XXX: set date */
 			MarkWholeScreenDirty();
 			break;
@@ -1021,11 +1081,104 @@ void SwitchToMode(SwitchMode new_mode)
 	}
 
 	if (_switch_mode_errorstr != INVALID_STRING_ID) {
-		ShowErrorMessage(INVALID_STRING_ID, _switch_mode_errorstr, 0, 0);
+		ShowErrorMessage(_switch_mode_errorstr, INVALID_STRING_ID, 0, 0, true);
 	}
 }
 
 
+/**
+ * Check the validity of some of the caches.
+ * Especially in the sense of desyncs between
+ * the cached value and what the value would
+ * be when calculated from the 'base' data.
+ */
+static void CheckCaches()
+{
+	/* Return here so it is easy to add checks that are run
+	 * always to aid testing of caches. */
+	if (_debug_desync_level <= 1) return;
+
+	/* Strict checking of the road stop cache entries */
+	const RoadStop *rs;
+	FOR_ALL_ROADSTOPS(rs) {
+		if (IsStandardRoadStopTile(rs->xy)) continue;
+
+		assert(rs->GetEntry(DIAGDIR_NE) != rs->GetEntry(DIAGDIR_NW));
+		rs->GetEntry(DIAGDIR_NE)->CheckIntegrity(rs);
+		rs->GetEntry(DIAGDIR_NW)->CheckIntegrity(rs);
+	}
+
+	Vehicle *v;
+	FOR_ALL_VEHICLES(v) {
+		if (v != v->First()) continue;
+
+		switch (v->type) {
+			case VEH_ROAD: {
+				RoadVehicle *rv = RoadVehicle::From(v);
+				RoadVehicleCache cache = rv->rcache;
+				RoadVehUpdateCache(rv);
+
+				if (memcmp(&cache, &rv->rcache, sizeof(RoadVehicleCache)) != 0) {
+					DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i\n", v->index, (int)v->owner, v->unitnumber);
+				}
+			} break;
+
+			case VEH_TRAIN: {
+				uint length = 0;
+				Train *t = Train::From(v);
+				for (Vehicle *u = t; u != NULL; u = u->Next()) length++;
+
+				TrainCache *wagons = MallocT<TrainCache>(length);
+				length = 0;
+				for (Train *u = t; u != NULL; u = u->Next()) wagons[length++] = u->tcache;
+
+				t->ConsistChanged(true);
+
+				length = 0;
+				for (Train *u = t; u != NULL; u = u->Next()) {
+					if (memcmp(&wagons[length], &u->tcache, sizeof(TrainCache)) != 0) {
+						DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i, wagon %i\n", v->index, (int)v->owner, v->unitnumber, length);
+					}
+					length++;
+				}
+
+				free(wagons);
+			} break;
+
+			case VEH_AIRCRAFT: {
+				Aircraft *a = Aircraft::From(v);
+				AircraftCache cache = a->acache;
+				UpdateAircraftCache(a);
+
+				if (memcmp(&cache, &a->acache, sizeof(AircraftCache)) != 0) {
+					DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i\n", v->index, (int)v->owner, v->unitnumber);
+				}
+			} break;
+
+			default:
+				break;
+		}
+	}
+
+	/* Check whether the caches are still valid */
+	FOR_ALL_VEHICLES(v) {
+		byte buff[sizeof(VehicleCargoList)];
+		memcpy(buff, &v->cargo, sizeof(VehicleCargoList));
+		v->cargo.InvalidateCache();
+		assert(memcmp(&v->cargo, buff, sizeof(VehicleCargoList)) == 0);
+	}
+
+	Station *st;
+	FOR_ALL_STATIONS(st) {
+		for (CargoID c = 0; c < NUM_CARGO; c++) {
+			byte buff[sizeof(StationCargoList)];
+			memcpy(buff, &st->goods[c].cargo, sizeof(StationCargoList));
+			st->goods[c].cargo.InvalidateCache();
+			assert(memcmp(&st->goods[c].cargo, buff, sizeof(StationCargoList)) == 0);
+		}
+	}
+}
+
 /**
  * State controlling game loop.
  * The state must not be changed from anywhere but here.
@@ -1034,7 +1187,7 @@ void SwitchToMode(SwitchMode new_mode)
 void StateGameLoop()
 {
 	/* dont execute the state loop during pause */
-	if (_pause_game) {
+	if (_pause_mode != PM_UNPAUSED) {
 		CallWindowTickEvent();
 		return;
 	}
@@ -1051,53 +1204,7 @@ void StateGameLoop()
 		CallWindowTickEvent();
 		NewsLoop();
 	} else {
-		if (_debug_desync_level > 1) {
-			Vehicle *v;
-			FOR_ALL_VEHICLES(v) {
-				if (v != v->First()) continue;
-
-				switch (v->type) {
-					case VEH_ROAD: {
-						extern byte GetRoadVehLength(const Vehicle *v);
-						if (GetRoadVehLength(v) != v->u.road.cached_veh_length) {
-							DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i\n", v->index, (int)v->owner, v->unitnumber);
-						}
-					} break;
-
-					case VEH_TRAIN: {
-						uint length = 0;
-						for (Vehicle *u = v; u != NULL; u = u->Next()) length++;
-
-						VehicleRail *wagons = MallocT<VehicleRail>(length);
-						length = 0;
-						for (Vehicle *u = v; u != NULL; u = u->Next()) wagons[length++] = u->u.rail;
-
-						TrainConsistChanged(v, true);
-
-						length = 0;
-						for (Vehicle *u = v; u != NULL; u = u->Next()) {
-							if (memcmp(&wagons[length], &u->u.rail, sizeof(VehicleRail)) != 0) {
-								DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i, wagon %i\n", v->index, (int)v->owner, v->unitnumber, length);
-							}
-							length++;
-						}
-
-						free(wagons);
-					} break;
-
-					case VEH_AIRCRAFT: {
-						uint speed = v->u.air.cached_max_speed;
-						UpdateAircraftCache(v);
-						if (speed != v->u.air.cached_max_speed) {
-							DEBUG(desync, 2, "cache mismatch: vehicle %i, company %i, unit number %i\n", v->index, (int)v->owner, v->unitnumber);
-						}
-					} break;
-
-					default:
-						break;
-				}
-			}
-		}
+		CheckCaches();
 
 		/* All these actions has to be done from OWNER_NONE
 		 *  for multiplayer compatibility */
@@ -1134,6 +1241,8 @@ static void DoAutosave()
 		GenerateDefaultSaveName(buf, lastof(buf));
 		strecat(buf, ".sav", lastof(buf));
 	} else {
+		static int _autosave_ctr = 0;
+
 		/* generate a savegame name and number according to _settings_client.gui.max_num_autosaves */
 		snprintf(buf, sizeof(buf), "autosave%d.sav", _autosave_ctr);
 
@@ -1142,7 +1251,7 @@ static void DoAutosave()
 
 	DEBUG(sl, 2, "Autosaving to '%s'", buf);
 	if (SaveOrLoad(buf, SL_SAVE, AUTOSAVE_DIR) != SL_OK) {
-		ShowErrorMessage(INVALID_STRING_ID, STR_AUTOSAVE_FAILED, 0, 0);
+		ShowErrorMessage(STR_ERROR_AUTOSAVE_FAILED, INVALID_STRING_ID, 0, 0);
 	}
 }
 
@@ -1157,9 +1266,6 @@ void GameLoop()
 		RedrawAutosave();
 	}
 
-	/* make a screenshot? */
-	if (IsScreenshotRequested()) ShowScreenshotResult(MakeScreenshot());
-
 	/* switch game mode? */
 	if (_switch_mode != SM_NONE) {
 		SwitchToMode(_switch_mode);
@@ -1185,8 +1291,7 @@ void GameLoop()
 		if (_network_reconnect > 0 && --_network_reconnect == 0) {
 			/* This means that we want to reconnect to the last host
 			 * We do this here, because it means that the network is really closed */
-			_network_playas = COMPANY_SPECTATOR;
-			NetworkClientConnectGame(NetworkAddress(_settings_client.network.last_host, _settings_client.network.last_port));
+			NetworkClientConnectGame(NetworkAddress(_settings_client.network.last_host, _settings_client.network.last_port), COMPANY_SPECTATOR);
 		}
 		/* Singleplayer */
 		StateGameLoop();
@@ -1195,9 +1300,9 @@ void GameLoop()
 	StateGameLoop();
 #endif /* ENABLE_NETWORK */
 
-	if (!_pause_game && HasBit(_display_opt, DO_FULL_ANIMATION)) DoPaletteAnimations();
+	if (!_pause_mode && HasBit(_display_opt, DO_FULL_ANIMATION)) DoPaletteAnimations();
 
-	if (!_pause_game || _cheats.build_in_pause.value) MoveAllTextEffects();
+	if (!_pause_mode || _cheats.build_in_pause.value) MoveAllTextEffects();
 
 	InputLoop();
 
