@@ -1,4 +1,11 @@
-/* $Id: oldloader_sl.cpp 16364 2009-05-21 11:47:45Z rubidium $ */
+/* $Id: oldloader_sl.cpp 19393 2010-03-12 21:12:35Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file oldloader_sl.cpp Chunks and fix-ups for TTO/TTD/TTDP savegames. TTO loader code is based on SVXConverter by Roman Vetter. */
 
@@ -11,22 +18,25 @@
 #include "../ship.h"
 #include "../train.h"
 #include "../signs_base.h"
+#include "../station_base.h"
+#include "../subsidy_base.h"
 #include "../debug.h"
 #include "../depot_base.h"
-#include "../newgrf_config.h"
-#include "../zoom_func.h"
 #include "../date_func.h"
 #include "../vehicle_func.h"
 #include "../variables.h"
 #include "../effectvehicle_base.h"
 #include "../core/mem_func.hpp"
 #include "../core/alloc_type.hpp"
+#include "../engine_base.h"
+#include "../engine_func.h"
+#include "../company_base.h"
 #include "saveload_internal.h"
 #include "oldloader.h"
 
 #include "table/strings.h"
 #include "../table/engines.h"
-#include "../table/namegen.h"
+#include "../table/townname.h"
 
 static bool   _read_ttdpatch_flags;
 
@@ -140,7 +150,7 @@ static uint32 RemapOldTownName(uint32 townnameparts, byte old_town_name_type)
 
 #undef FIXNUM
 
-void FixOldTowns()
+static void FixOldTowns()
 {
 	Town *town;
 
@@ -153,13 +163,19 @@ void FixOldTowns()
 	}
 }
 
-StringID *_old_vehicle_names;
+static StringID *_old_vehicle_names;
 
 void FixOldVehicles()
 {
 	Vehicle *v;
 
 	FOR_ALL_VEHICLES(v) {
+		if ((size_t)v->next == 0xFFFF) {
+			v->next = NULL;
+		} else {
+			v->next = Vehicle::GetIfValid((size_t)v->next);
+		}
+
 		/* For some reason we need to correct for this */
 		switch (v->spritenum) {
 			case 0xfd: break;
@@ -173,10 +189,11 @@ void FixOldVehicles()
 		v->name = CopyFromOldName(_old_vehicle_names[v->index]);
 
 		/* We haven't used this bit for stations for ages */
-		if (v->type == VEH_ROAD &&
-				v->u.road.state != RVSB_IN_DEPOT &&
-				v->u.road.state != RVSB_WORMHOLE) {
-			ClrBit(v->u.road.state, RVS_IS_STOPPING);
+		if (v->type == VEH_ROAD) {
+			RoadVehicle *rv = RoadVehicle::From(v);
+			if (rv->state != RVSB_IN_DEPOT && rv->state != RVSB_WORMHOLE) {
+				ClrBit(rv->state, 2);
+			}
 		}
 
 		/* The subtype should be 0, but it sometimes isn't :( */
@@ -199,18 +216,20 @@ static bool FixTTOMapArray()
 {
 	for (TileIndex t = 0; t < OLD_MAP_SIZE; t++) {
 		TileType tt = GetTileType(t);
+		if (tt == 11) {
+			/* TTO has a different way of storing monorail.
+			 * Instead of using bits in m3 it uses a different tile type. */
+			_m[t].m3 = 1; // rail type = monorail (in TTD)
+			SetTileType(t, MP_RAILWAY);
+			_m[t].m2 = 1; // set monorail ground to RAIL_GROUND_GRASS
+			tt = MP_RAILWAY;
+		}
 
 		switch (tt) {
 			case MP_CLEAR:
 				break;
 
 			case MP_RAILWAY:
-			case 11: // monorail
-				if (tt == 11) {
-					_m[t].m3 = 1; // rail type = monorail
-					_m[t].type_height &= 0x1F; // -> MP_RAILWAY
-					_m[t].m2 = 1; // set monorail ground to RAIL_GROUND_GRASS
-				}
 				switch (GB(_m[t].m5, 6, 2)) {
 					case 0: // RAIL_TILE_NORMAL
 						break;
@@ -336,7 +355,7 @@ static bool FixTTOEngines()
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255,  44,  45,  46, 255, 255, 255, 255,  47,  48, 255,  49,  50,
 		255, 255, 255, 255,  51,  52, 255,  53,  54, 255,  55,  56, 255,  57,  58, 255,
-		 59,  60,   255,61,  62, 255,  63,  64, 255,  65,  66, 255, 255, 255, 255, 255,
+		 59,  60, 255,  61,  62, 255,  63,  64, 255,  65,  66, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
 		255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,  67,  68,  69,  70,
@@ -402,7 +421,6 @@ static bool FixTTOEngines()
 			e->duration_phase_1    = oe->duration_phase_1;
 			e->duration_phase_2    = oe->duration_phase_2;
 			e->duration_phase_3    = oe->duration_phase_3;
-			e->lifelength          = oe->lifelength;
 			e->flags               = oe->flags;
 
 			e->company_avail = 0;
@@ -417,12 +435,12 @@ static bool FixTTOEngines()
 				}
 			}
 
-			e->preview_company_rank = 0;
-			e->preview_wait = 0;
-			e->name = NULL;
-
 			e->info.climates = 1;
 		}
+
+		e->preview_company_rank = 0;
+		e->preview_wait = 0;
+		e->name = NULL;
 	}
 
 	return true;
@@ -485,16 +503,16 @@ static void ReadTTDPatchFlags()
 	/* TTDPatch misuses _old_map3 for flags.. read them! */
 	_old_vehicle_multiplier = _old_map3[0];
 	/* Somehow.... there was an error in some savegames, so 0 becomes 1
-	and 1 becomes 2. The rest of the values are okay */
+	 * and 1 becomes 2. The rest of the values are okay */
 	if (_old_vehicle_multiplier < 2) _old_vehicle_multiplier++;
 
 	_old_vehicle_names = MallocT<StringID>(_old_vehicle_multiplier * 850);
 
 	/* TTDPatch increases the Vehicle-part in the middle of the game,
-	so if the multipler is anything else but 1, the assert fails..
-	bump the assert value so it doesn't!
-	(1 multipler == 850 vehicles
-	1 vehicle   == 128 bytes */
+	 * so if the multipler is anything else but 1, the assert fails..
+	 * bump the assert value so it doesn't!
+	 * (1 multipler == 850 vehicles
+	 * 1 vehicle   == 128 bytes */
 	_bump_assert_value = (_old_vehicle_multiplier - 1) * 850 * 128;
 
 	for (uint i = 0; i < 17; i++) { // check tile 0, too
@@ -520,36 +538,36 @@ static const OldChunks town_chunk[] = {
 	OCL_NULL( 2 ),         ///< population,        no longer in use
 	OCL_SVAR( OC_UINT16, Town, townnametype ),
 	OCL_SVAR( OC_UINT32, Town, townnameparts ),
-	OCL_SVAR(  OC_UINT8, Town, grow_counter ),
+	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Town, grow_counter ),
 	OCL_NULL( 1 ),         ///< sort_index,        no longer in use
 	OCL_NULL( 4 ),         ///< sign-coordinates,  no longer in use
 	OCL_NULL( 2 ),         ///< namewidth,         no longer in use
-	OCL_SVAR( OC_UINT16, Town, flags12 ),
+	OCL_SVAR( OC_FILE_U16 |  OC_VAR_U8, Town, flags ),
 	OCL_NULL( 10 ),        ///< radius,            no longer in use
 
-	OCL_SVAR( OC_UINT16, Town, ratings[0] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[1] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[2] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[3] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[4] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[5] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[6] ),
-	OCL_SVAR( OC_UINT16, Town, ratings[7] ),
+	OCL_SVAR( OC_INT16, Town, ratings[0] ),
+	OCL_SVAR( OC_INT16, Town, ratings[1] ),
+	OCL_SVAR( OC_INT16, Town, ratings[2] ),
+	OCL_SVAR( OC_INT16, Town, ratings[3] ),
+	OCL_SVAR( OC_INT16, Town, ratings[4] ),
+	OCL_SVAR( OC_INT16, Town, ratings[5] ),
+	OCL_SVAR( OC_INT16, Town, ratings[6] ),
+	OCL_SVAR( OC_INT16, Town, ratings[7] ),
 
 	OCL_SVAR( OC_FILE_U32 | OC_VAR_U16, Town, have_ratings ),
 	OCL_SVAR( OC_FILE_U32 | OC_VAR_U16, Town, statues ),
 	OCL_NULL( 2 ),         ///< num_houses,        no longer in use
-	OCL_SVAR(  OC_UINT8, Town, time_until_rebuild ),
-	OCL_SVAR(  OC_UINT8, Town, growth_rate ),
+	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Town, time_until_rebuild ),
+	OCL_SVAR(  OC_FILE_U8 | OC_VAR_I16, Town, growth_rate ),
 
-	OCL_SVAR( OC_UINT16, Town, new_max_pass ),
-	OCL_SVAR( OC_UINT16, Town, new_max_mail ),
-	OCL_SVAR( OC_UINT16, Town, new_act_pass ),
-	OCL_SVAR( OC_UINT16, Town, new_act_mail ),
-	OCL_SVAR( OC_UINT16, Town, max_pass ),
-	OCL_SVAR( OC_UINT16, Town, max_mail ),
-	OCL_SVAR( OC_UINT16, Town, act_pass ),
-	OCL_SVAR( OC_UINT16, Town, act_mail ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, new_max_pass ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, new_max_mail ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, new_act_pass ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, new_act_mail ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, max_pass ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, max_mail ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, act_pass ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Town, act_mail ),
 
 	OCL_SVAR(  OC_UINT8, Town, pct_pass_transported ),
 	OCL_SVAR(  OC_UINT8, Town, pct_mail_transported ),
@@ -578,7 +596,7 @@ static bool LoadOldTown(LoadgameState *ls, int num)
 			t->townnametype = t->townnametype == 0x10B6 ? 0x20C1 : t->townnametype + 0x2A00;
 		}
 	} else {
-		t->xy = INVALID_TILE;
+		delete t;
 	}
 
 	return true;
@@ -594,12 +612,16 @@ static bool LoadOldOrder(LoadgameState *ls, int num)
 {
 	if (!LoadChunk(ls, NULL, order_chunk)) return false;
 
-	new (num) Order(UnpackOldOrder(_old_order));
+	Order *o = new (num) Order();
+	o->AssignOrder(UnpackOldOrder(_old_order));
 
-	/* Relink the orders to eachother (in the orders for one vehicle are behind eachother,
-	 * with an invalid order (OT_NOTHING) as indication that it is the last order */
-	if (num > 0 && GetOrder(num)->IsValid()) {
-		GetOrder(num - 1)->next = GetOrder(num);
+	if (o->IsType(OT_NOTHING)) {
+		delete o;
+	} else {
+		/* Relink the orders to eachother (in the orders for one vehicle are behind eachother,
+		 * with an invalid order (OT_NOTHING) as indication that it is the last order */
+		Order *prev = Order::GetIfValid(num - 1);
+		if (prev != NULL) prev->next = o;
 	}
 
 	return true;
@@ -642,82 +664,12 @@ static bool LoadOldDepot(LoadgameState *ls, int num)
 	if (d->xy != 0) {
 		d->town_index = RemapTownIndex(_old_town_index);
 	} else {
-		d->xy = INVALID_TILE;
+		delete d;
 	}
 
 	return true;
 }
 
-static int32 _old_price;
-static uint16 _old_price_frac;
-static const OldChunks price_chunk[] = {
-	OCL_VAR (  OC_INT32,   1, &_old_price ),
-	OCL_VAR ( OC_UINT16,   1, &_old_price_frac ),
-	OCL_END()
-};
-
-static bool LoadOldPrice(LoadgameState *ls, int num)
-{
-	if (_savegame_type == SGT_TTO && num == 25) {
-		/* clear_fields == build_road_depot (TTO didn't have this price) */
-		((Money*)&_price)[25] = ((Money*)&_price)[6];
-		_price_frac[25] = _price_frac[6];
-		return true;
-	}
-
-	if (!LoadChunk(ls, NULL, price_chunk)) return false;
-
-	if (_savegame_type == SGT_TTO) {
-		/* base prices are different in these two cases */
-		if (num == 15) _old_price = ClampToI32(((Money)_old_price) * 20 / 3); // build_railvehicle
-		if (num == 17) _old_price = ClampToI32(((Money)_old_price) * 10);     // aircraft_base
-	}
-
-
-	/* We use a struct to store the prices, but they are ints in a row..
-	 * so just access the struct as an array of int32s */
-	((Money*)&_price)[num] = _old_price;
-	_price_frac[num] = _old_price_frac;
-
-	return true;
-}
-
-static const OldChunks cargo_payment_rate_chunk[] = {
-	OCL_VAR (  OC_INT32,   1, &_old_price ),
-	OCL_VAR ( OC_UINT16,   1, &_old_price_frac ),
-
-	OCL_NULL( 2 ),         ///< Junk
-	OCL_END()
-};
-
-static bool LoadOldCargoPaymentRate(LoadgameState *ls, int num)
-{
-	if (_savegame_type == SGT_TTO && num == 11) { // TTD has 1 more cargo type
-		_cargo_payment_rates[num] = _cargo_payment_rates[9];
-		_cargo_payment_rates_frac[num] = _cargo_payment_rates_frac[9];
-		return true;
-	}
-
-	if (!LoadChunk(ls, NULL, cargo_payment_rate_chunk)) return false;
-
-	if (_savegame_type == SGT_TTO) {
-		/* SVXConverter about cargo payment rates correction:
-		 * "increase them to compensate for the faster time advance in TTD compared to TTO
-		 * which otherwise would cause much less income while the annual running costs of
-		 * the vehicles stay the same" */
-
-		Money m = ((((Money)_old_price) << 16) + (uint)_old_price_frac) * 124 / 74;
-
-		_old_price = m >> 16;
-		_old_price_frac = GB((int64)m, 0, 16);
-	}
-
-	_cargo_payment_rates[num] = -_old_price;
-	_cargo_payment_rates_frac[num] = _old_price_frac;
-
-	return true;
-}
-
 static StationID _current_station_id;
 static uint16 _waiting_acceptance;
 static uint8  _cargo_source;
@@ -740,7 +692,7 @@ static bool LoadOldGood(LoadgameState *ls, int num)
 	/* for TTO games, 12th (num == 11) goods entry is created in the Station constructor */
 	if (_savegame_type == SGT_TTO && num == 11) return true;
 
-	Station *st = GetStation(_current_station_id);
+	Station *st = Station::Get(_current_station_id);
 	GoodsEntry *ge = &st->goods[num];
 
 	if (!LoadChunk(ls, ge, goods_chunk)) return false;
@@ -748,11 +700,7 @@ static bool LoadOldGood(LoadgameState *ls, int num)
 	SB(ge->acceptance_pickup, GoodsEntry::ACCEPTANCE, 1, HasBit(_waiting_acceptance, 15));
 	SB(ge->acceptance_pickup, GoodsEntry::PICKUP, 1, _cargo_source != 0xFF);
 	if (GB(_waiting_acceptance, 0, 12) != 0) {
-		CargoPacket *cp = new CargoPacket();
-		cp->source          = (_cargo_source == 0xFF) ? INVALID_STATION : _cargo_source;
-		cp->count           = GB(_waiting_acceptance, 0, 12);
-		cp->days_in_transit = _cargo_days;
-		ge->cargo.Append(cp);
+		ge->cargo.Append(new CargoPacket(GB(_waiting_acceptance, 0, 12), _cargo_days, (_cargo_source == 0xFF) ? INVALID_STATION : _cargo_source, 0, 0));
 	}
 
 	return true;
@@ -763,10 +711,10 @@ static const OldChunks station_chunk[] = {
 	OCL_VAR ( OC_UINT32,   1, &_old_town_index ),
 
 	OCL_NULL( 4 ), ///< bus/lorry tile
-	OCL_SVAR(   OC_TILE, Station, train_tile ),
+	OCL_SVAR(   OC_TILE, Station, train_station.tile ),
 	OCL_SVAR(   OC_TILE, Station, airport_tile ),
 	OCL_SVAR(   OC_TILE, Station, dock_tile ),
-	OCL_SVAR(  OC_UINT8, Station, trainst_w ),
+	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Station, train_station.w ),
 
 	OCL_NULL( 1 ),         ///< sort-index, no longer in use
 	OCL_NULL( 2 ),         ///< sign-width, no longer in use
@@ -775,7 +723,7 @@ static const OldChunks station_chunk[] = {
 
 	OCL_NULL( 4 ),         ///< sign left/top, no longer in use
 
-	OCL_SVAR( OC_UINT16, Station, had_vehicle_of_type ),
+	OCL_SVAR( OC_FILE_U16 | OC_VAR_U8, Station, had_vehicle_of_type ),
 
 	OCL_CHUNK( 12, LoadOldGood ),
 
@@ -803,7 +751,7 @@ static bool LoadOldStation(LoadgameState *ls, int num)
 	if (!LoadChunk(ls, st, station_chunk)) return false;
 
 	if (st->xy != 0) {
-		st->town = GetTown(RemapTownIndex(_old_town_index));
+		st->town = Town::Get(RemapTownIndex(_old_town_index));
 
 		if (_savegame_type == SGT_TTO) {
 			if (IsInsideBS(_old_string_id, 0x180F, 32)) {
@@ -823,17 +771,17 @@ static bool LoadOldStation(LoadgameState *ls, int num)
 			st->string_id = RemapOldStringID(_old_string_id);
 		}
 	} else {
-		st->xy = INVALID_TILE;
+		delete st;
 	}
 
 	return true;
 }
 
 static const OldChunks industry_chunk[] = {
-	OCL_SVAR(   OC_TILE, Industry, xy ),
+	OCL_SVAR(   OC_TILE, Industry, location.tile ),
 	OCL_VAR ( OC_UINT32,   1, &_old_town_index ),
-	OCL_SVAR(  OC_UINT8, Industry, width ),
-	OCL_SVAR(  OC_UINT8, Industry, height ),
+	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Industry, location.w ),
+	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Industry, location.h ),
 	OCL_NULL( 2 ),  ///< used to be industry's produced_cargo
 
 	OCL_SVAR( OC_TTD | OC_UINT16, Industry, produced_cargo_waiting[0] ),
@@ -879,8 +827,8 @@ static bool LoadOldIndustry(LoadgameState *ls, int num)
 	Industry *i = new (num) Industry();
 	if (!LoadChunk(ls, i, industry_chunk)) return false;
 
-	if (i->xy != 0) {
-		i->town = GetTown(RemapTownIndex(_old_town_index));
+	if (i->location.tile != 0) {
+		i->town = Town::Get(RemapTownIndex(_old_town_index));
 
 		if (_savegame_type == SGT_TTO) {
 			if (i->type > 0x06) i->type++; // Printing Works were added
@@ -895,7 +843,7 @@ static bool LoadOldIndustry(LoadgameState *ls, int num)
 
 		IncIndustryTypeCount(i->type);
 	} else {
-		i->xy = INVALID_TILE;
+		delete i;
 	}
 
 	return true;
@@ -911,7 +859,7 @@ static const OldChunks _company_yearly_chunk[] = {
 
 static bool LoadOldCompanyYearly(LoadgameState *ls, int num)
 {
-	Company *c = GetCompany(_current_company_id);
+	Company *c = Company::Get(_current_company_id);
 
 	for (uint i = 0; i < 13; i++) {
 		if (_savegame_type == SGT_TTO && i == 6) {
@@ -938,7 +886,7 @@ static const OldChunks _company_economy_chunk[] = {
 
 static bool LoadOldCompanyEconomy(LoadgameState *ls, int num)
 {
-	Company *c = GetCompany(_current_company_id);
+	Company *c = Company::Get(_current_company_id);
 
 	if (!LoadChunk(ls, &c->cur_economy, _company_economy_chunk)) return false;
 
@@ -968,8 +916,8 @@ static const OldChunks _company_chunk[] = {
 
 	OCL_SVAR(  OC_UINT8, Company, colour ),
 	OCL_SVAR(  OC_UINT8, Company, money_fraction ),
-	OCL_SVAR(  OC_UINT8, Company, quarters_of_bankrupcy ),
-	OCL_SVAR(  OC_UINT8, Company, bankrupt_asked ),
+	OCL_SVAR(  OC_UINT8, Company, quarters_of_bankruptcy ),
+	OCL_SVAR( OC_FILE_U8  | OC_VAR_U16, Company, bankrupt_asked ),
 	OCL_SVAR( OC_FILE_U32 | OC_VAR_I64, Company, bankrupt_value ),
 	OCL_SVAR( OC_UINT16, Company, bankrupt_timeout ),
 
@@ -1073,10 +1021,10 @@ static uint16 _old_next_ptr;
 static VehicleID _current_vehicle_id;
 
 static const OldChunks vehicle_train_chunk[] = {
-	OCL_SVAR(  OC_UINT8, VehicleRail, track ),
-	OCL_SVAR(  OC_UINT8, VehicleRail, force_proceed ),
-	OCL_SVAR( OC_UINT16, VehicleRail, crash_anim_pos ),
-	OCL_SVAR(  OC_UINT8, VehicleRail, railtype ),
+	OCL_SVAR(  OC_UINT8, Train, track ),
+	OCL_SVAR(  OC_UINT8, Train, force_proceed ),
+	OCL_SVAR( OC_UINT16, Train, crash_anim_pos ),
+	OCL_SVAR(  OC_UINT8, Train, railtype ),
 
 	OCL_NULL( 5 ), ///< Junk
 
@@ -1084,13 +1032,13 @@ static const OldChunks vehicle_train_chunk[] = {
 };
 
 static const OldChunks vehicle_road_chunk[] = {
-	OCL_SVAR(  OC_UINT8, VehicleRoad, state ),
-	OCL_SVAR(  OC_UINT8, VehicleRoad, frame ),
-	OCL_SVAR( OC_UINT16, VehicleRoad, blocked_ctr ),
-	OCL_SVAR(  OC_UINT8, VehicleRoad, overtaking ),
-	OCL_SVAR(  OC_UINT8, VehicleRoad, overtaking_ctr ),
-	OCL_SVAR( OC_UINT16, VehicleRoad, crashed_ctr ),
-	OCL_SVAR(  OC_UINT8, VehicleRoad, reverse_ctr ),
+	OCL_SVAR(  OC_UINT8, RoadVehicle, state ),
+	OCL_SVAR(  OC_UINT8, RoadVehicle, frame ),
+	OCL_SVAR( OC_UINT16, RoadVehicle, blocked_ctr ),
+	OCL_SVAR(  OC_UINT8, RoadVehicle, overtaking ),
+	OCL_SVAR(  OC_UINT8, RoadVehicle, overtaking_ctr ),
+	OCL_SVAR( OC_UINT16, RoadVehicle, crashed_ctr ),
+	OCL_SVAR(  OC_UINT8, RoadVehicle, reverse_ctr ),
 
 	OCL_NULL( 1 ), ///< Junk
 
@@ -1098,7 +1046,7 @@ static const OldChunks vehicle_road_chunk[] = {
 };
 
 static const OldChunks vehicle_ship_chunk[] = {
-	OCL_SVAR(  OC_UINT8, VehicleShip, state ),
+	OCL_SVAR(  OC_UINT8, Ship, state ),
 
 	OCL_NULL( 9 ), ///< Junk
 
@@ -1106,10 +1054,10 @@ static const OldChunks vehicle_ship_chunk[] = {
 };
 
 static const OldChunks vehicle_air_chunk[] = {
-	OCL_SVAR(  OC_UINT8, VehicleAir, pos ),
-	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, VehicleAir, targetairport ),
-	OCL_SVAR( OC_UINT16, VehicleAir, crashed_counter ),
-	OCL_SVAR(  OC_UINT8, VehicleAir, state ),
+	OCL_SVAR(  OC_UINT8, Aircraft, pos ),
+	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Aircraft, targetairport ),
+	OCL_SVAR( OC_UINT16, Aircraft, crashed_counter ),
+	OCL_SVAR(  OC_UINT8, Aircraft, state ),
 
 	OCL_NULL( 5 ), ///< Junk
 
@@ -1117,8 +1065,8 @@ static const OldChunks vehicle_air_chunk[] = {
 };
 
 static const OldChunks vehicle_effect_chunk[] = {
-	OCL_SVAR( OC_UINT16, VehicleEffect, animation_state ),
-	OCL_SVAR(  OC_UINT8, VehicleEffect, animation_substate ),
+	OCL_SVAR( OC_UINT16, EffectVehicle, animation_state ),
+	OCL_SVAR(  OC_UINT8, EffectVehicle, animation_substate ),
 
 	OCL_NULL( 7 ), // Junk
 
@@ -1126,8 +1074,8 @@ static const OldChunks vehicle_effect_chunk[] = {
 };
 
 static const OldChunks vehicle_disaster_chunk[] = {
-	OCL_SVAR( OC_UINT16, VehicleDisaster, image_override ),
-	OCL_SVAR( OC_UINT16, VehicleDisaster, big_ufo_destroyer_target ),
+	OCL_SVAR( OC_UINT16, DisasterVehicle, image_override ),
+	OCL_SVAR( OC_UINT16, DisasterVehicle, big_ufo_destroyer_target ),
 
 	OCL_NULL( 6 ), ///< Junk
 
@@ -1142,19 +1090,22 @@ static const OldChunks vehicle_empty_chunk[] = {
 
 static bool LoadOldVehicleUnion(LoadgameState *ls, int num)
 {
-	Vehicle *v = GetVehicle(_current_vehicle_id);
+	Vehicle *v = Vehicle::GetIfValid(_current_vehicle_id);
 	uint temp = ls->total_read;
 	bool res;
 
-	switch (v->type) {
-		default: NOT_REACHED();
-		case VEH_INVALID : res = LoadChunk(ls, NULL,           vehicle_empty_chunk);    break;
-		case VEH_TRAIN   : res = LoadChunk(ls, &v->u.rail,     vehicle_train_chunk);    break;
-		case VEH_ROAD    : res = LoadChunk(ls, &v->u.road,     vehicle_road_chunk);     break;
-		case VEH_SHIP    : res = LoadChunk(ls, &v->u.ship,     vehicle_ship_chunk);     break;
-		case VEH_AIRCRAFT: res = LoadChunk(ls, &v->u.air,      vehicle_air_chunk);      break;
-		case VEH_EFFECT  : res = LoadChunk(ls, &v->u.effect,   vehicle_effect_chunk);   break;
-		case VEH_DISASTER: res = LoadChunk(ls, &v->u.disaster, vehicle_disaster_chunk); break;
+	if (v == NULL) {
+		res = LoadChunk(ls, NULL, vehicle_empty_chunk);
+	} else {
+		switch (v->type) {
+			default: NOT_REACHED();
+			case VEH_TRAIN   : res = LoadChunk(ls, v, vehicle_train_chunk);    break;
+			case VEH_ROAD    : res = LoadChunk(ls, v, vehicle_road_chunk);     break;
+			case VEH_SHIP    : res = LoadChunk(ls, v, vehicle_ship_chunk);     break;
+			case VEH_AIRCRAFT: res = LoadChunk(ls, v, vehicle_air_chunk);      break;
+			case VEH_EFFECT  : res = LoadChunk(ls, v, vehicle_effect_chunk);   break;
+			case VEH_DISASTER: res = LoadChunk(ls, v, vehicle_disaster_chunk); break;
+		}
 	}
 
 	/* This chunk size should always be 10 bytes */
@@ -1180,7 +1131,7 @@ static const OldChunks vehicle_chunk[] = {
 	OCL_NULL ( 1 ), ///< num_orders, now calculated
 	OCL_SVAR(  OC_UINT8, Vehicle, cur_order_index ),
 	OCL_SVAR(   OC_TILE, Vehicle, dest_tile ),
-	OCL_SVAR( OC_UINT16, Vehicle, load_unload_time_rem ),
+	OCL_SVAR( OC_UINT16, Vehicle, load_unload_ticks ),
 	OCL_SVAR( OC_FILE_U16 | OC_VAR_U32, Vehicle, date_of_last_service ),
 	OCL_SVAR( OC_UINT16, Vehicle, service_interval ),
 	OCL_SVAR( OC_FILE_U8 | OC_VAR_U16, Vehicle, last_station_visited ),
@@ -1271,7 +1222,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			uint type = ReadByte(ls);
 			switch (type) {
 				default: return false;
-				case 0x00 /* VEH_INVALID  */: v = new (_current_vehicle_id) InvalidVehicle();  break;
+				case 0x00 /* VEH_INVALID  */: v = NULL;                                        break;
 				case 0x25 /* MONORAIL     */:
 				case 0x20 /* VEH_TRAIN    */: v = new (_current_vehicle_id) Train();           break;
 				case 0x21 /* VEH_ROAD     */: v = new (_current_vehicle_id) RoadVehicle();     break;
@@ -1282,6 +1233,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			}
 
 			if (!LoadChunk(ls, v, vehicle_chunk)) return false;
+			if (v == NULL) continue;
 
 			SpriteID sprite = v->cur_image;
 			/* no need to override other sprites */
@@ -1305,7 +1257,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 					};
 					if (v->spritenum / 2 >= lengthof(spriteset_rail)) return false;
 					v->spritenum = spriteset_rail[v->spritenum / 2]; // adjust railway sprite set offset
-					v->u.rail.railtype = type == 0x25 ? 1 : 0; // monorail / rail
+					Train::From(v)->railtype = type == 0x25 ? 1 : 0; // monorail / rail
 					break;
 				}
 
@@ -1334,12 +1286,12 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 
 			switch (_old_string_id) {
 				case 0x0000: break; // empty (invalid vehicles)
-				case 0x0006: _old_string_id  = STR_SV_EMPTY;         break; // empty (special vehicles)
-				case 0x8495: _old_string_id  = STR_SV_TRAIN_NAME;    break; // "Train X"
-				case 0x8842: _old_string_id  = STR_SV_ROADVEH_NAME;  break; // "Road Vehicle X"
-				case 0x8C3B: _old_string_id  = STR_SV_SHIP_NAME;     break; // "Ship X"
-				case 0x9047: _old_string_id  = STR_SV_AIRCRAFT_NAME; break; // "Aircraft X"
-				default:     _old_string_id += 0x2A00;               break; // custom name
+				case 0x0006: _old_string_id  = STR_SV_EMPTY;              break; // empty (special vehicles)
+				case 0x8495: _old_string_id  = STR_SV_TRAIN_NAME;         break; // "Train X"
+				case 0x8842: _old_string_id  = STR_SV_ROAD_VEHICLE_NAME;  break; // "Road Vehicle X"
+				case 0x8C3B: _old_string_id  = STR_SV_SHIP_NAME;          break; // "Ship X"
+				case 0x9047: _old_string_id  = STR_SV_AIRCRAFT_NAME;      break; // "Aircraft X"
+				default:     _old_string_id += 0x2A00;                    break; // custom name
 			}
 
 			_old_vehicle_names[_current_vehicle_id] = _old_string_id;
@@ -1347,7 +1299,7 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 			/* Read the vehicle type and allocate the right vehicle */
 			switch (ReadByte(ls)) {
 				default: NOT_REACHED();
-				case 0x00 /* VEH_INVALID */: v = new (_current_vehicle_id) InvalidVehicle();  break;
+				case 0x00 /* VEH_INVALID */: v = NULL;                                        break;
 				case 0x10 /* VEH_TRAIN   */: v = new (_current_vehicle_id) Train();           break;
 				case 0x11 /* VEH_ROAD    */: v = new (_current_vehicle_id) RoadVehicle();     break;
 				case 0x12 /* VEH_SHIP    */: v = new (_current_vehicle_id) Ship();            break;
@@ -1355,7 +1307,9 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 				case 0x14 /* VEH_EFFECT  */: v = new (_current_vehicle_id) EffectVehicle();   break;
 				case 0x15 /* VEH_DISASTER*/: v = new (_current_vehicle_id) DisasterVehicle(); break;
 			}
+
 			if (!LoadChunk(ls, v, vehicle_chunk)) return false;
+			if (v == NULL) continue;
 
 			_old_vehicle_names[_current_vehicle_id] = RemapOldStringID(_old_string_id);
 
@@ -1369,16 +1323,16 @@ bool LoadOldVehicle(LoadgameState *ls, int num)
 		if (_old_order_ptr != 0 && _old_order_ptr != 0xFFFFFFFF) {
 			uint max = _savegame_type == SGT_TTO ? 3000 : 5000;
 			uint old_id = RemapOrderIndex(_old_order_ptr);
-			if (old_id < max) v->orders.old = GetOrder(old_id); // don't accept orders > max number of orders
+			if (old_id < max) v->orders.old = Order::Get(old_id); // don't accept orders > max number of orders
 		}
 		v->current_order.AssignOrder(UnpackOldOrder(_old_order));
 
-		if (_old_next_ptr != 0xFFFF) v->next = GetVehiclePoolSize() <= _old_next_ptr ? new (_old_next_ptr) InvalidVehicle() : GetVehicle(_old_next_ptr);
+		v->next = (Vehicle *)(size_t)_old_next_ptr;
 
 		if (_cargo_count != 0) {
-			CargoPacket *cp = new CargoPacket((_cargo_source == 0xFF) ? INVALID_STATION : _cargo_source, _cargo_count);
-			cp->days_in_transit = _cargo_days;
-			v->cargo.Append(cp);
+			StationID source =    (_cargo_source == 0xFF) ? INVALID_STATION : _cargo_source;
+			TileIndex source_xy = (source != INVALID_STATION) ? Station::Get(source)->xy : 0;
+			v->cargo.Append(new CargoPacket(_cargo_count, _cargo_days, source, source_xy, source_xy));
 		}
 	}
 
@@ -1408,6 +1362,8 @@ static bool LoadOldSign(LoadgameState *ls, int num)
 			si->name = CopyFromOldName(RemapOldStringID(_old_string_id));
 		}
 		si->owner = OWNER_NONE;
+	} else {
+		delete si;
 	}
 
 	return true;
@@ -1426,7 +1382,7 @@ static const OldChunks engine_chunk[] = {
 	OCL_SVAR( OC_UINT16, Engine, duration_phase_2 ),
 	OCL_SVAR( OC_UINT16, Engine, duration_phase_3 ),
 
-	OCL_SVAR(  OC_UINT8, Engine, lifelength ),
+	OCL_NULL( 1 ), // lifelength
 	OCL_SVAR(  OC_UINT8, Engine, flags ),
 	OCL_SVAR(  OC_UINT8, Engine, preview_company_rank ),
 	OCL_SVAR(  OC_UINT8, Engine, preview_wait ),
@@ -1451,16 +1407,19 @@ static bool LoadOldEngineName(LoadgameState *ls, int num)
 
 static const OldChunks subsidy_chunk[] = {
 	OCL_SVAR(  OC_UINT8, Subsidy, cargo_type ),
-	OCL_SVAR(  OC_UINT8, Subsidy, age ),
-	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Subsidy, from ),
-	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Subsidy, to ),
+	OCL_SVAR(  OC_UINT8, Subsidy, remaining ),
+	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Subsidy, src ),
+	OCL_SVAR(  OC_FILE_U8 | OC_VAR_U16, Subsidy, dst ),
 
 	OCL_END()
 };
 
 static bool LoadOldSubsidy(LoadgameState *ls, int num)
 {
-	return LoadChunk(ls, &_subsidies[num], subsidy_chunk);
+	Subsidy *s = new (num) Subsidy();
+	bool ret = LoadChunk(ls, s, subsidy_chunk);
+	if (s->cargo_type == CT_INVALID) delete s;
+	return ret;
 }
 
 static const OldChunks game_difficulty_chunk[] = {
@@ -1638,11 +1597,13 @@ static const OldChunks main_chunk[] = {
 
 	OCL_ASSERT( OC_TTO, 0x3A2E ),
 
-	OCL_CHUNK( 49, LoadOldPrice ),
+	OCL_CNULL( OC_TTO, 48 * 6 ), ///< prices
+	OCL_CNULL( OC_TTD, 49 * 6 ), ///< prices
 
 	OCL_ASSERT( OC_TTO, 0x3B4E ),
 
-	OCL_CHUNK( 12, LoadOldCargoPaymentRate ),
+	OCL_CNULL( OC_TTO, 11 * 8 ), ///< cargo payment rates
+	OCL_CNULL( OC_TTD, 12 * 8 ), ///< cargo payment rates
 
 	OCL_ASSERT( OC_TTD, 0x4CBA ),
 	OCL_ASSERT( OC_TTO, 0x3BA6 ),
@@ -1703,19 +1664,19 @@ static const OldChunks main_chunk[] = {
 	OCL_VAR ( OC_FILE_I16 | OC_VAR_I32,   1, &_saved_scrollpos_y ),
 	OCL_VAR ( OC_FILE_U16 | OC_VAR_U8,    1, &_saved_scrollpos_zoom ),
 
-	OCL_VAR ( OC_FILE_U32 | OC_VAR_I64,   1, &_economy.max_loan ),
-	OCL_VAR ( OC_FILE_U32 | OC_VAR_I64,   1, &_economy.max_loan_unround ),
+	OCL_NULL( 4 ),           ///< max_loan
+	OCL_VAR ( OC_FILE_U32 | OC_VAR_I64,   1, &_economy.old_max_loan_unround ),
 	OCL_VAR (  OC_INT16,    1, &_economy.fluct ),
 
 	OCL_VAR ( OC_UINT16,    1, &_disaster_delay ),
 
 	OCL_ASSERT( OC_TTO, 0x496E4 ),
 
-	OCL_CNULL( OC_TTD, 144 ),             ///< cargo-stuff, calculated in InitializeLandscapeVariables
+	OCL_CNULL( OC_TTD, 144 ),             ///< cargo-stuff
 
 	OCL_CCHUNK( OC_TTD, 256, LoadOldEngineName ),
 
-	OCL_CNULL( OC_TTD, 144 ),             ///< AI cargo-stuff, calculated in InitializeLandscapeVariables
+	OCL_CNULL( OC_TTD, 144 ),             ///< AI cargo-stuff
 	OCL_NULL( 2 ),               ///< Company indexes of companies, no longer in use
 	OCL_NULL( 1 ),               ///< Station tick counter, no longer in use
 
@@ -1746,7 +1707,7 @@ static const OldChunks main_chunk[] = {
 	OCL_VAR ( OC_TTD | OC_UINT8,    1, &_settings_game.game_creation.snow_line ),
 
 	OCL_CNULL( OC_TTD, 32 ),              ///< new_industry_randtable, no longer used (because of new design)
-	OCL_CNULL( OC_TTD, 36 ),              ///< cargo-stuff, calculated in InitializeLandscapeVariables
+	OCL_CNULL( OC_TTD, 36 ),              ///< cargo-stuff
 
 	OCL_ASSERT( OC_TTD, 0x77179 ),
 	OCL_ASSERT( OC_TTO, 0x4971D ),
@@ -1833,6 +1794,12 @@ bool LoadTTOMain(LoadgameState *ls)
 	/* We have a new difficulty setting */
 	_settings_game.difficulty.town_council_tolerance = Clamp(_settings_game.difficulty.diff_level, 0, 2);
 
+	/* SVXConverter about cargo payment rates correction:
+	 * "increase them to compensate for the faster time advance in TTD compared to TTO
+	 * which otherwise would cause much less income while the annual running costs of
+	 * the vehicles stay the same" */
+	_economy.inflation_payment = min(_economy.inflation_payment * 124 / 74, MAX_INFLATION);
+
 	DEBUG(oldloader, 3, "Finished converting game data");
 	DEBUG(oldloader, 1, "TTO savegame successfully converted");
 
