@@ -1,4 +1,11 @@
-/* $Id: ship_cmd.cpp 16482 2009-05-31 12:22:53Z rubidium $ */
+/* $Id: ship_cmd.cpp 19091 2010-02-10 21:28:26Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file ship_cmd.cpp Handling of ships. */
 
@@ -7,14 +14,14 @@
 #include "landscape.h"
 #include "timetable.h"
 #include "command_func.h"
-#include "station_map.h"
 #include "news_func.h"
 #include "company_func.h"
-#include "npf.h"
+#include "pathfinder/npf/npf_func.h"
 #include "depot_base.h"
+#include "station_base.h"
 #include "vehicle_gui.h"
 #include "newgrf_engine.h"
-#include "yapf/yapf.h"
+#include "pathfinder/yapf/yapf.h"
 #include "newgrf_sound.h"
 #include "spritecache.h"
 #include "strings_func.h"
@@ -23,26 +30,21 @@
 #include "date_func.h"
 #include "vehicle_func.h"
 #include "sound_func.h"
-#include "variables.h"
 #include "autoreplace_gui.h"
-#include "gfx_func.h"
 #include "effectvehicle_func.h"
-#include "settings_type.h"
+#include "effectvehicle_base.h"
 #include "ai/ai.hpp"
-#include "pathfind.h"
+#include "pathfinder/opf/opf_ship.h"
+#include "landscape_type.h"
+#include "engine_base.h"
+#include "engine_func.h"
+#include "company_base.h"
 
 #include "table/strings.h"
 #include "table/sprites.h"
 
 static const uint16 _ship_sprites[] = {0x0E5D, 0x0E55, 0x0E65, 0x0E6D};
 
-static const TrackBits _ship_sometracks[4] = {
-	TRACK_BIT_X | TRACK_BIT_LOWER | TRACK_BIT_LEFT,  // 0x19, // DIAGDIR_NE
-	TRACK_BIT_Y | TRACK_BIT_UPPER | TRACK_BIT_LEFT,  // 0x16, // DIAGDIR_SE
-	TRACK_BIT_X | TRACK_BIT_UPPER | TRACK_BIT_RIGHT, // 0x25, // DIAGDIR_SW
-	TRACK_BIT_Y | TRACK_BIT_LOWER | TRACK_BIT_RIGHT, // 0x2A, // DIAGDIR_NW
-};
-
 static inline TrackBits GetTileShipTrackStatus(TileIndex tile)
 {
 	return TrackStatusToTrackBits(GetTileTrackStatus(tile, TRANSPORT_WATER, 0));
@@ -50,21 +52,25 @@ static inline TrackBits GetTileShipTrackStatus(TileIndex tile)
 
 static SpriteID GetShipIcon(EngineID engine)
 {
-	uint8 spritenum = ShipVehInfo(engine)->image_index;
+	const Engine *e = Engine::Get(engine);
+	uint8 spritenum = e->u.ship.image_index;
 
 	if (is_custom_sprite(spritenum)) {
 		SpriteID sprite = GetCustomVehicleIcon(engine, DIR_W);
 		if (sprite != 0) return sprite;
 
-		spritenum = GetEngine(engine)->image_index;
+		spritenum = e->original_image_index;
 	}
 
-	return 6 + _ship_sprites[spritenum];
+	return DIR_W + _ship_sprites[spritenum];
 }
 
-void DrawShipEngine(int x, int y, EngineID engine, SpriteID pal)
+void DrawShipEngine(int left, int right, int preferred_x, int y, EngineID engine, PaletteID pal)
 {
-	DrawSprite(GetShipIcon(engine), pal, x, y);
+	SpriteID sprite = GetShipIcon(engine);
+	const Sprite *real_sprite = GetSprite(sprite, ST_NORMAL);
+	preferred_x = Clamp(preferred_x, left - real_sprite->x_offs, right - real_sprite->width - real_sprite->x_offs);
+	DrawSprite(sprite, pal, preferred_x, y);
 }
 
 /** Get the size of the sprite of a ship sprite heading west (used for lists)
@@ -88,28 +94,23 @@ SpriteID Ship::GetImage(Direction direction) const
 		SpriteID sprite = GetCustomVehicleSprite(this, direction);
 		if (sprite != 0) return sprite;
 
-		spritenum = GetEngine(this->engine_type)->image_index;
+		spritenum = Engine::Get(this->engine_type)->original_image_index;
 	}
 
 	return _ship_sprites[spritenum] + direction;
 }
 
-static const Depot *FindClosestShipDepot(const Vehicle *v)
+static const Depot *FindClosestShipDepot(const Vehicle *v, uint max_distance)
 {
-	if (_settings_game.pf.pathfinder_for_ships == VPF_NPF) { // NPF is used
-		Trackdir trackdir = GetVehicleTrackdir(v);
-		NPFFoundTargetData ftd = NPFRouteToDepotTrialError(v->tile, trackdir, false, TRANSPORT_WATER, 0, v->owner, INVALID_RAILTYPES);
-
-		if (ftd.best_bird_dist == 0) return GetDepotByTile(ftd.node.tile); // Found target
-
-		return NULL; // Did not find target
-	}
-
-	/* OPF or YAPF - find the closest depot */
-
+	/* Find the closest depot */
 	const Depot *depot;
 	const Depot *best_depot = NULL;
-	uint best_dist = UINT_MAX;
+	/* If we don't have a maximum distance, i.e. distance = 0,
+	 * we want to find any depot so the best distance of no
+	 * depot must be more than any correct distance. On the
+	 * other hand if we have set a maximum distance, any depot
+	 * further away than max_distance can safely be ignored. */
+	uint best_dist = max_distance == 0 ? UINT_MAX : max_distance + 1;
 
 	FOR_ALL_DEPOTS(depot) {
 		TileIndex tile = depot->xy;
@@ -127,30 +128,40 @@ static const Depot *FindClosestShipDepot(const Vehicle *v)
 
 static void CheckIfShipNeedsService(Vehicle *v)
 {
-	if (_settings_game.vehicle.servint_ships == 0 || !v->NeedsAutomaticServicing()) return;
+	if (Company::Get(v->owner)->settings.vehicle.servint_ships == 0 || !v->NeedsAutomaticServicing()) return;
 	if (v->IsInDepot()) {
 		VehicleServiceInDepot(v);
 		return;
 	}
 
-	const Depot *depot = FindClosestShipDepot(v);
+	uint max_distance;
+	switch (_settings_game.pf.pathfinder_for_ships) {
+		case VPF_OPF:  max_distance = 12; break;
+		case VPF_NPF:  max_distance = _settings_game.pf.npf.maximum_go_to_depot_penalty  / NPF_TILE_LENGTH;  break;
+		case VPF_YAPF: max_distance = _settings_game.pf.yapf.maximum_go_to_depot_penalty / YAPF_TILE_LENGTH; break;
+		default: NOT_REACHED();
+	}
 
-	if (depot == NULL || DistanceManhattan(v->tile, depot->xy) > 12) {
+	const Depot *depot = FindClosestShipDepot(v, max_distance);
+
+	if (depot == NULL) {
 		if (v->current_order.IsType(OT_GOTO_DEPOT)) {
 			v->current_order.MakeDummy();
-			InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
+			SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
 		}
 		return;
 	}
 
 	v->current_order.MakeGoToDepot(depot->index, ODTFB_SERVICE);
 	v->dest_tile = depot->xy;
-	InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
+	SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
 }
 
 Money Ship::GetRunningCost() const
 {
-	return GetVehicleProperty(this, 0x0F, ShipVehInfo(this->engine_type)->running_cost) * _price.ship_running;
+	const Engine *e = Engine::Get(this->engine_type);
+	uint cost_factor = GetVehicleProperty(this, PROP_SHIP_RUNNING_COST_FACTOR, e->u.ship.running_cost);
+	return GetPrice(PR_RUNNING_SHIP, cost_factor, e->grffile);
 }
 
 void Ship::OnNewDay()
@@ -173,9 +184,26 @@ void Ship::OnNewDay()
 
 	SubtractMoneyFromCompanyFract(this->owner, cost);
 
-	InvalidateWindow(WC_VEHICLE_DETAILS, this->index);
+	SetWindowDirty(WC_VEHICLE_DETAILS, this->index);
 	/* we need this for the profit */
-	InvalidateWindowClasses(WC_SHIPS_LIST);
+	SetWindowClassesDirty(WC_SHIPS_LIST);
+}
+
+Trackdir Ship::GetVehicleTrackdir() const
+{
+	if (this->vehstatus & VS_CRASHED) return INVALID_TRACKDIR;
+
+	if (this->IsInDepot()) {
+		/* We'll assume the ship is facing outwards */
+		return DiagDirToDiagTrackdir(GetShipDepotDirection(this->tile));
+	}
+
+	if (this->state == TRACK_BIT_WORMHOLE) {
+		/* ship on aqueduct, so just use his direction and assume a diagonal track */
+		return DiagDirToDiagTrackdir(DirToDiagDir(this->direction));
+	}
+
+	return TrackDirectionToTrackdir(FindFirstTrack(this->state), this->direction);
 }
 
 static void HandleBrokenShip(Vehicle *v)
@@ -187,8 +215,9 @@ static void HandleBrokenShip(Vehicle *v)
 		if (v->breakdowns_since_last_service != 255)
 			v->breakdowns_since_last_service++;
 
-		InvalidateWindow(WC_VEHICLE_VIEW, v->index);
-		InvalidateWindow(WC_VEHICLE_DETAILS, v->index);
+		v->MarkDirty();
+		SetWindowDirty(WC_VEHICLE_VIEW, v->index);
+		SetWindowDirty(WC_VEHICLE_DETAILS, v->index);
 
 		if (!PlayVehicleSound(v, VSE_BREAKDOWN)) {
 			SndPlayVehicleFx((_settings_game.game_creation.landscape != LT_TOYLAND) ?
@@ -196,23 +225,23 @@ static void HandleBrokenShip(Vehicle *v)
 		}
 
 		if (!(v->vehstatus & VS_HIDDEN)) {
-			Vehicle *u = CreateEffectVehicleRel(v, 4, 4, 5, EV_BREAKDOWN_SMOKE);
-			if (u != NULL) u->u.effect.animation_state = v->breakdown_delay * 2;
+			EffectVehicle *u = CreateEffectVehicleRel(v, 4, 4, 5, EV_BREAKDOWN_SMOKE);
+			if (u != NULL) u->animation_state = v->breakdown_delay * 2;
 		}
 	}
 
 	if (!(v->tick_counter & 1)) {
 		if (!--v->breakdown_delay) {
 			v->breakdown_ctr = 0;
-			InvalidateWindow(WC_VEHICLE_VIEW, v->index);
+			v->MarkDirty();
+			SetWindowDirty(WC_VEHICLE_VIEW, v->index);
 		}
 	}
 }
 
 void Ship::MarkDirty()
 {
-	this->cur_image = this->GetImage(this->direction);
-	MarkSingleVehicleDirty(this);
+	this->UpdateViewport(false, false);
 }
 
 static void PlayShipSound(const Vehicle *v)
@@ -231,11 +260,11 @@ TileIndex Ship::GetOrderStationLocation(StationID station)
 {
 	if (station == this->last_station_visited) this->last_station_visited = INVALID_STATION;
 
-	const Station *st = GetStation(station);
+	const Station *st = Station::Get(station);
 	if (st->dock_tile != INVALID_TILE) {
 		return TILE_ADD(st->dock_tile, ToTileIndexDiff(GetDockOffset(st->dock_tile)));
 	} else {
-		this->cur_order_index++;
+		this->IncrementOrderIndex();
 		return 0;
 	}
 }
@@ -265,10 +294,8 @@ void Ship::UpdateDeltaXY(Direction direction)
 
 void RecalcShipStuff(Vehicle *v)
 {
-	v->UpdateDeltaXY(v->direction);
-	v->cur_image = v->GetImage(v->direction);
-	v->MarkDirty();
-	InvalidateWindow(WC_VEHICLE_DEPOT, v->tile);
+	v->UpdateViewport(false, true);
+	SetWindowDirty(WC_VEHICLE_DEPOT, v->tile);
 }
 
 static const TileIndexDiffC _ship_leave_depot_offs[] = {
@@ -276,7 +303,7 @@ static const TileIndexDiffC _ship_leave_depot_offs[] = {
 	{ 0, -1}
 };
 
-static void CheckShipLeaveDepot(Vehicle *v)
+static void CheckShipLeaveDepot(Ship *v)
 {
 	if (!v->IsInDepot()) return;
 
@@ -284,16 +311,16 @@ static void CheckShipLeaveDepot(Vehicle *v)
 	Axis axis = GetShipDepotAxis(tile);
 
 	/* Check first (north) side */
-	if (_ship_sometracks[axis] & GetTileShipTrackStatus(TILE_ADD(tile, ToTileIndexDiff(_ship_leave_depot_offs[axis])))) {
+	if (DiagdirReachesTracks((DiagDirection)axis) & GetTileShipTrackStatus(TILE_ADD(tile, ToTileIndexDiff(_ship_leave_depot_offs[axis])))) {
 		v->direction = ReverseDir(AxisToDirection(axis));
 	/* Check second (south) side */
-	} else if (_ship_sometracks[axis + 2] & GetTileShipTrackStatus(TILE_ADD(tile, -2 * ToTileIndexDiff(_ship_leave_depot_offs[axis])))) {
+	} else if (DiagdirReachesTracks((DiagDirection)(axis + 2)) & GetTileShipTrackStatus(TILE_ADD(tile, -2 * ToTileIndexDiff(_ship_leave_depot_offs[axis])))) {
 		v->direction = AxisToDirection(axis);
 	} else {
 		return;
 	}
 
-	v->u.ship.state = AxisToTrackBits(axis);
+	v->state = AxisToTrackBits(axis);
 	v->vehstatus &= ~VS_HIDDEN;
 
 	v->cur_speed = 0;
@@ -302,7 +329,7 @@ static void CheckShipLeaveDepot(Vehicle *v)
 	PlayShipSound(v);
 	VehicleServiceInDepot(v);
 	InvalidateWindowData(WC_VEHICLE_DEPOT, v->tile);
-	InvalidateWindowClasses(WC_SHIPS_LIST);
+	SetWindowClassesDirty(WC_SHIPS_LIST);
 }
 
 static bool ShipAccelerate(Vehicle *v)
@@ -310,13 +337,13 @@ static bool ShipAccelerate(Vehicle *v)
 	uint spd;
 	byte t;
 
-	spd = min(v->cur_speed + 1, GetVehicleProperty(v, 0x0B, v->max_speed));
+	spd = min(v->cur_speed + 1, GetVehicleProperty(v, PROP_SHIP_SPEED, v->max_speed));
 
 	/* updates statusbar only if speed have changed to save CPU time */
 	if (spd != v->cur_speed) {
 		v->cur_speed = spd;
 		if (_settings_client.gui.vehicle_speed)
-			InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
+			SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
 	}
 
 	/* Decrease somewhat when turning */
@@ -337,8 +364,8 @@ static void ShipArrivesAt(const Vehicle *v, Station *st)
 		st->had_vehicle_of_type |= HVOT_SHIP;
 
 		SetDParam(0, st->index);
-		AddNewsItem(
-			STR_9833_CITIZENS_CELEBRATE_FIRST,
+		AddVehicleNewsItem(
+			STR_NEWS_FIRST_SHIP_ARRIVAL,
 			(v->owner == _local_company) ? NS_ARRIVAL_COMPANY : NS_ARRIVAL_OTHER,
 			v->index,
 			st->index
@@ -347,154 +374,20 @@ static void ShipArrivesAt(const Vehicle *v, Station *st)
 	}
 }
 
-struct PathFindShip {
-	TileIndex skiptile;
-	TileIndex dest_coords;
-	uint best_bird_dist;
-	uint best_length;
-};
-
-static bool ShipTrackFollower(TileIndex tile, PathFindShip *pfs, int track, uint length)
-{
-	/* Found dest? */
-	if (tile == pfs->dest_coords) {
-		pfs->best_bird_dist = 0;
-
-		pfs->best_length = minu(pfs->best_length, length);
-		return true;
-	}
-
-	/* Skip this tile in the calculation */
-	if (tile != pfs->skiptile) {
-		pfs->best_bird_dist = minu(pfs->best_bird_dist, DistanceMaxPlusManhattan(pfs->dest_coords, tile));
-	}
-
-	return false;
-}
-
-static const byte _ship_search_directions[6][4] = {
-	{ 0, 9, 2, 9 },
-	{ 9, 1, 9, 3 },
-	{ 9, 0, 3, 9 },
-	{ 1, 9, 9, 2 },
-	{ 3, 2, 9, 9 },
-	{ 9, 9, 1, 0 },
-};
-
-static const byte _pick_shiptrack_table[6] = {1, 3, 2, 2, 0, 0};
-
-static uint FindShipTrack(Vehicle *v, TileIndex tile, DiagDirection dir, TrackBits bits, TileIndex skiptile, Track *track)
-{
-	PathFindShip pfs;
-	Track i, best_track;
-	uint best_bird_dist = 0;
-	uint best_length    = 0;
-	uint r;
-	byte ship_dir = v->direction & 3;
-
-	pfs.dest_coords = v->dest_tile;
-	pfs.skiptile = skiptile;
-
-	best_track = INVALID_TRACK;
-
-	do {
-		i = RemoveFirstTrack(&bits);
-
-		pfs.best_bird_dist = UINT_MAX;
-		pfs.best_length = UINT_MAX;
-
-		FollowTrack(tile, PATHFIND_FLAGS_SHIP_MODE | PATHFIND_FLAGS_DISABLE_TILE_HASH, TRANSPORT_WATER, 0, (DiagDirection)_ship_search_directions[i][dir], (TPFEnumProc*)ShipTrackFollower, NULL, &pfs);
-
-		if (best_track != INVALID_TRACK) {
-			if (pfs.best_bird_dist != 0) {
-				/* neither reached the destination, pick the one with the smallest bird dist */
-				if (pfs.best_bird_dist > best_bird_dist) goto bad;
-				if (pfs.best_bird_dist < best_bird_dist) goto good;
-			} else {
-				if (pfs.best_length > best_length) goto bad;
-				if (pfs.best_length < best_length) goto good;
-			}
-
-			/* if we reach this position, there's two paths of equal value so far.
-			 * pick one randomly. */
-			r = GB(Random(), 0, 8);
-			if (_pick_shiptrack_table[i] == ship_dir) r += 80;
-			if (_pick_shiptrack_table[best_track] == ship_dir) r -= 80;
-			if (r <= 127) goto bad;
-		}
-good:;
-		best_track = i;
-		best_bird_dist = pfs.best_bird_dist;
-		best_length = pfs.best_length;
-bad:;
-
-	} while (bits != 0);
-
-	*track = best_track;
-	return best_bird_dist;
-}
-
-static inline NPFFoundTargetData PerfNPFRouteToStationOrTile(TileIndex tile, Trackdir trackdir, bool ignore_start_tile, NPFFindStationOrTileData *target, TransportType type, Owner owner, RailTypes railtypes)
-{
-
-	void *perf = NpfBeginInterval();
-	NPFFoundTargetData ret = NPFRouteToStationOrTile(tile, trackdir, ignore_start_tile, target, type, 0, owner, railtypes);
-	int t = NpfEndInterval(perf);
-	DEBUG(yapf, 4, "[NPFW] %d us - %d rounds - %d open - %d closed -- ", t, 0, _aystar_stats_open_size, _aystar_stats_closed_size);
-	return ret;
-}
 
 /** returns the track to choose on the next tile, or -1 when it's better to
  * reverse. The tile given is the tile we are about to enter, enterdir is the
  * direction in which we are entering the tile */
-static Track ChooseShipTrack(Vehicle *v, TileIndex tile, DiagDirection enterdir, TrackBits tracks)
+static Track ChooseShipTrack(const Ship *v, TileIndex tile, DiagDirection enterdir, TrackBits tracks)
 {
 	assert(IsValidDiagDirection(enterdir));
 
 	switch (_settings_game.pf.pathfinder_for_ships) {
-		case VPF_YAPF: { // YAPF
-			Trackdir trackdir = YapfChooseShipTrack(v, tile, enterdir, tracks);
-			if (trackdir != INVALID_TRACKDIR) return TrackdirToTrack(trackdir);
-		} break;
-
-		case VPF_NPF: { // NPF
-			NPFFindStationOrTileData fstd;
-			Trackdir trackdir = GetVehicleTrackdir(v);
-			assert(trackdir != INVALID_TRACKDIR); // Check that we are not in a depot
-
-			NPFFillWithOrderData(&fstd, v);
-
-			NPFFoundTargetData ftd = PerfNPFRouteToStationOrTile(tile - TileOffsByDiagDir(enterdir), trackdir, true, &fstd, TRANSPORT_WATER, v->owner, INVALID_RAILTYPES);
-
-			/* If ftd.best_bird_dist is 0, we found our target and ftd.best_trackdir contains
-			 * the direction we need to take to get there, if ftd.best_bird_dist is not 0,
-			 * we did not find our target, but ftd.best_trackdir contains the direction leading
-			 * to the tile closest to our target. */
-			if (ftd.best_trackdir != 0xff) return TrackdirToTrack(ftd.best_trackdir); // TODO: Wrapper function?
-		} break;
-
-		default:
-		case VPF_OPF: { // OPF
-			TileIndex tile2 = TILE_ADD(tile, -TileOffsByDiagDir(enterdir));
-			Track track;
-
-			/* Let's find out how far it would be if we would reverse first */
-			TrackBits b = GetTileShipTrackStatus(tile2) & _ship_sometracks[ReverseDiagDir(enterdir)] & v->u.ship.state;
-
-			uint distr = UINT_MAX; // distance if we reversed
-			if (b != 0) {
-				distr = FindShipTrack(v, tile2, ReverseDiagDir(enterdir), b, tile, &track);
-				if (distr != UINT_MAX) distr++; // penalty for reversing
-			}
-
-			/* And if we would not reverse? */
-			uint dist = FindShipTrack(v, tile, enterdir, tracks, 0, &track);
-
-			if (dist <= distr) return track;
-		} break;
+		case VPF_OPF: return OPFShipChooseTrack(v, tile, enterdir, tracks);
+		case VPF_NPF: return NPFShipChooseTrack(v, tile, enterdir, tracks);
+		case VPF_YAPF: return YapfShipChooseTrack(v, tile, enterdir, tracks);
+		default: NOT_REACHED();
 	}
-
-	return INVALID_TRACK; // We could better reverse
 }
 
 static const Direction _new_vehicle_direction_table[] = {
@@ -518,9 +411,9 @@ static Direction ShipGetNewDirection(Vehicle *v, int x, int y)
 	return _new_vehicle_direction_table[offs];
 }
 
-static inline TrackBits GetAvailShipTracks(TileIndex tile, int dir)
+static inline TrackBits GetAvailShipTracks(TileIndex tile, DiagDirection dir)
 {
-	return GetTileShipTrackStatus(tile) & _ship_sometracks[dir];
+	return GetTileShipTrackStatus(tile) & DiagdirReachesTracks(dir);
 }
 
 static const byte _ship_subcoord[4][6][3] = {
@@ -558,7 +451,7 @@ static const byte _ship_subcoord[4][6][3] = {
 	}
 };
 
-static void ShipController(Vehicle *v)
+static void ShipController(Ship *v)
 {
 	uint32 r;
 	const byte *b;
@@ -589,7 +482,7 @@ static void ShipController(Vehicle *v)
 	if (!ShipAccelerate(v)) return;
 
 	GetNewVehiclePosResult gp = GetNewVehiclePos(v);
-	if (v->u.ship.state != TRACK_BIT_WORMHOLE) {
+	if (v->state != TRACK_BIT_WORMHOLE) {
 		/* Not on a bridge */
 		if (gp.old_tile == gp.new_tile) {
 			/* Staying in tile */
@@ -605,18 +498,16 @@ static void ShipController(Vehicle *v)
 				 * always skip ahead. */
 				if (v->current_order.IsType(OT_LEAVESTATION)) {
 					v->current_order.Free();
-					InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
+					SetWindowWidgetDirty(WC_VEHICLE_VIEW, v->index, VVW_WIDGET_START_STOP_VEH);
 				} else if (v->dest_tile != 0) {
 					/* We have a target, let's see if we reached it... */
-					if (v->current_order.IsType(OT_GOTO_STATION) &&
-							GetStation(v->current_order.GetDestination())->IsBuoy() &&
+					if (v->current_order.IsType(OT_GOTO_WAYPOINT) &&
 							DistanceManhattan(v->dest_tile, gp.new_tile) <= 3) {
 						/* We got within 3 tiles of our target buoy, so let's skip to our
 						 * next order */
 						UpdateVehicleTimetable(v, true);
-						v->cur_order_index++;
+						v->IncrementOrderIndex();
 						v->current_order.MakeDummy();
-						InvalidateVehicleOrder(v, 0);
 					} else {
 						/* Non-buoy orders really need to reach the tile */
 						if (v->dest_tile == gp.new_tile) {
@@ -629,14 +520,13 @@ static void ShipController(Vehicle *v)
 								v->last_station_visited = v->current_order.GetDestination();
 
 								/* Process station in the orderlist. */
-								Station *st = GetStation(v->current_order.GetDestination());
+								Station *st = Station::Get(v->current_order.GetDestination());
 								if (st->facilities & FACIL_DOCK) { // ugly, ugly workaround for problem with ships able to drop off cargo at wrong stations
 									ShipArrivesAt(v, st);
 									v->BeginLoading();
 								} else { // leave stations without docks right aways
 									v->current_order.MakeLeaveStation();
-									v->cur_order_index++;
-									InvalidateVehicleOrder(v, 0);
+									v->IncrementOrderIndex();
 								}
 							}
 						}
@@ -671,7 +561,7 @@ static void ShipController(Vehicle *v)
 
 			if (!HasBit(r, VETS_ENTERED_WORMHOLE)) {
 				v->tile = gp.new_tile;
-				v->u.ship.state = TrackToTrackBits(track);
+				v->state = TrackToTrackBits(track);
 			}
 
 			v->direction = (Direction)b[2];
@@ -693,9 +583,7 @@ static void ShipController(Vehicle *v)
 	v->z_pos = GetSlopeZ(gp.x, gp.y);
 
 getout:
-	v->UpdateDeltaXY(dir);
-	v->cur_image = v->GetImage(dir);
-	VehicleMove(v, true);
+	v->UpdateViewport(true, true);
 	return;
 
 reverse_direction:
@@ -704,18 +592,13 @@ reverse_direction:
 	goto getout;
 }
 
-static void AgeShipCargo(Vehicle *v)
-{
-	if (_age_cargo_skip_counter != 0) return;
-	v->cargo.AgeCargo();
-}
-
-void Ship::Tick()
+bool Ship::Tick()
 {
 	if (!(this->vehstatus & VS_STOPPED)) this->running_ticks++;
 
-	AgeShipCargo(this);
 	ShipController(this);
+
+	return true;
 }
 
 /** Build a ship.
@@ -723,14 +606,16 @@ void Ship::Tick()
  * @param flags type of operation
  * @param p1 ship type being built (engine)
  * @param p2 unused
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
 	UnitID unit_num;
 
-	if (!IsEngineBuildable(p1, VEH_SHIP, _current_company)) return_cmd_error(STR_SHIP_NOT_AVAILABLE);
+	if (!IsEngineBuildable(p1, VEH_SHIP, _current_company)) return_cmd_error(STR_ERROR_SHIP_NOT_AVAILABLE);
 
-	const Engine *e = GetEngine(p1);
+	const Engine *e = Engine::Get(p1);
 	CommandCost value(EXPENSES_NEW_VEHICLES, e->GetCost());
 
 	/* Engines without valid cargo should not be available */
@@ -746,15 +631,15 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 	unit_num = (flags & DC_AUTOREPLACE) ? 0 : GetFreeUnitNumber(VEH_SHIP);
 
 	if (!Vehicle::CanAllocateItem() || unit_num > _settings_game.vehicle.max_ships)
-		return_cmd_error(STR_00E1_TOO_MANY_VEHICLES_IN_GAME);
+		return_cmd_error(STR_ERROR_TOO_MANY_VEHICLES_IN_GAME);
 
 	if (flags & DC_EXEC) {
 		int x;
 		int y;
 
-		const ShipVehicleInfo *svi = ShipVehInfo(p1);
+		const ShipVehicleInfo *svi = &e->u.ship;
 
-		Vehicle *v = new Ship();
+		Ship *v = new Ship();
 		v->unitnumber = unit_num;
 
 		v->owner = _current_company;
@@ -765,14 +650,11 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 		v->y_pos = y;
 		v->z_pos = GetSlopeZ(x, y);
 
-		v->running_ticks = 0;
-
 		v->UpdateDeltaXY(v->direction);
 		v->vehstatus = VS_HIDDEN | VS_STOPPED | VS_DEFPAL;
 
 		v->spritenum = svi->image_index;
 		v->cargo_type = e->GetDefaultCargoType();
-		v->cargo_subtype = 0;
 		v->cargo_cap = svi->capacity;
 		v->value = value.GetCost();
 
@@ -782,24 +664,22 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 
 		v->reliability = e->reliability;
 		v->reliability_spd_dec = e->reliability_spd_dec;
-		v->max_age = e->lifelength * DAYS_IN_LEAP_YEAR;
+		v->max_age = e->GetLifeLengthInDays();
 		_new_vehicle_id = v->index;
 
-		v->name = NULL;
-		v->u.ship.state = TRACK_BIT_DEPOT;
+		v->state = TRACK_BIT_DEPOT;
 
-		v->service_interval = _settings_game.vehicle.servint_ships;
+		v->service_interval = Company::Get(_current_company)->settings.vehicle.servint_ships;
 		v->date_of_last_service = _date;
 		v->build_year = _cur_year;
-		v->cur_image = 0x0E5E;
+		v->cur_image = SPR_IMG_QUERY;
 		v->random_bits = VehicleRandomBits();
 
-		v->vehicle_flags = 0;
 		if (e->flags & ENGINE_EXCLUSIVE_PREVIEW) SetBit(v->vehicle_flags, VF_BUILT_AS_PROTOTYPE);
 
 		v->InvalidateNewGRFCacheOfChain();
 
-		v->cargo_cap = GetVehicleProperty(v, 0x0D, svi->capacity);
+		v->cargo_cap = GetVehicleCapacity(v);
 
 		v->InvalidateNewGRFCacheOfChain();
 
@@ -807,11 +687,12 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 
 		InvalidateWindowData(WC_VEHICLE_DEPOT, v->tile);
 		InvalidateWindowClassesData(WC_SHIPS_LIST, 0);
-		InvalidateWindow(WC_COMPANY, v->owner);
-		if (IsLocalCompany())
+		SetWindowDirty(WC_COMPANY, v->owner);
+		if (IsLocalCompany()) {
 			InvalidateAutoreplaceWindow(v->engine_type, v->group_id); // updates the replace Ship window
+		}
 
-		GetCompany(_current_company)->num_engines[p1]++;
+		Company::Get(_current_company)->num_engines[p1]++;
 	}
 
 	return value;
@@ -822,21 +703,18 @@ CommandCost CmdBuildShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
  * @param flags type of operation
  * @param p1 vehicle ID to be sold
  * @param p2 unused
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdSellShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	Vehicle *v;
+	Ship *v = Ship::GetIfValid(p1);
+	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
 
-	if (!IsValidVehicleID(p1)) return CMD_ERROR;
-
-	v = GetVehicle(p1);
-
-	if (v->type != VEH_SHIP || !CheckOwnership(v->owner)) return CMD_ERROR;
-
-	if (HASBITS(v->vehstatus, VS_CRASHED)) return_cmd_error(STR_CAN_T_SELL_DESTROYED_VEHICLE);
+	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_ERROR_CAN_T_SELL_DESTROYED_VEHICLE);
 
 	if (!v->IsStoppedInDepot()) {
-		return_cmd_error(STR_980B_SHIP_MUST_BE_STOPPED_IN);
+		return_cmd_error(STR_ERROR_SHIP_MUST_BE_STOPPED_IN_DEPOT);
 	}
 
 	CommandCost ret(EXPENSES_NEW_VEHICLES, -v->value);
@@ -850,7 +728,7 @@ CommandCost CmdSellShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p
 
 bool Ship::FindClosestDepot(TileIndex *location, DestinationID *destination, bool *reverse)
 {
-	const Depot *depot = FindClosestShipDepot(this);
+	const Depot *depot = FindClosestShipDepot(this, 0);
 
 	if (depot == NULL) return false;
 
@@ -867,6 +745,8 @@ bool Ship::FindClosestDepot(TileIndex *location, DestinationID *destination, boo
  * @param p2 various bitmasked elements
  * - p2 bit 0-3 - DEPOT_ flags (see vehicle.h)
  * - p2 bit 8-10 - VLW flag (for mass goto depot)
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdSendShipToDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
@@ -876,11 +756,8 @@ CommandCost CmdSendShipToDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 		return SendAllVehiclesToDepot(VEH_SHIP, flags, p2 & DEPOT_SERVICE, _current_company, (p2 & VLW_MASK), p1);
 	}
 
-	if (!IsValidVehicleID(p1)) return CMD_ERROR;
-
-	Vehicle *v = GetVehicle(p1);
-
-	if (v->type != VEH_SHIP) return CMD_ERROR;
+	Ship *v = Ship::GetIfValid(p1);
+	if (v == NULL) return CMD_ERROR;
 
 	return v->SendToDepot(flags, (DepotCommand)(p2 & DEPOT_COMMAND_MASK));
 }
@@ -894,63 +771,32 @@ CommandCost CmdSendShipToDepot(TileIndex tile, DoCommandFlag flags, uint32 p1, u
  * - p2 = (bit 0-7) - the new cargo type to refit to (p2 & 0xFF)
  * - p2 = (bit 8-15) - the new cargo subtype to refit to
  * - p2 = (bit 16) - refit only this vehicle (ignored)
- * @return cost of refit or error
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdRefitShip(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	Vehicle *v;
-	CommandCost cost(EXPENSES_SHIP_RUN);
 	CargoID new_cid = GB(p2, 0, 8); // gets the cargo number
 	byte new_subtype = GB(p2, 8, 8);
-	uint16 capacity = CALLBACK_FAILED;
 
-	if (!IsValidVehicleID(p1)) return CMD_ERROR;
+	Ship *v = Ship::GetIfValid(p1);
 
-	v = GetVehicle(p1);
-
-	if (v->type != VEH_SHIP || !CheckOwnership(v->owner)) return CMD_ERROR;
-	if (!v->IsStoppedInDepot()) return_cmd_error(STR_980B_SHIP_MUST_BE_STOPPED_IN);
-	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_CAN_T_REFIT_DESTROYED_VEHICLE);
+	if (v == NULL || !CheckOwnership(v->owner)) return CMD_ERROR;
+	if (!v->IsStoppedInDepot()) return_cmd_error(STR_ERROR_SHIP_MUST_BE_STOPPED_IN_DEPOT);
+	if (v->vehstatus & VS_CRASHED) return_cmd_error(STR_ERROR_CAN_T_REFIT_DESTROYED_VEHICLE);
 
 	/* Check cargo */
-	if (!ShipVehInfo(v->engine_type)->refittable) return CMD_ERROR;
-	if (new_cid >= NUM_CARGO || !CanRefitTo(v->engine_type, new_cid)) return CMD_ERROR;
+	if (new_cid >= NUM_CARGO) return CMD_ERROR;
 
-	/* Check the refit capacity callback */
-	if (HasBit(EngInfo(v->engine_type)->callbackmask, CBM_VEHICLE_REFIT_CAPACITY)) {
-		/* Back up the existing cargo type */
-		CargoID temp_cid = v->cargo_type;
-		byte temp_subtype = v->cargo_subtype;
-		v->cargo_type = new_cid;
-		v->cargo_subtype = new_subtype;
-
-		capacity = GetVehicleCallback(CBID_VEHICLE_REFIT_CAPACITY, 0, 0, v->engine_type, v);
-
-		/* Restore the cargo type */
-		v->cargo_type = temp_cid;
-		v->cargo_subtype = temp_subtype;
-	}
-
-	if (capacity == CALLBACK_FAILED) {
-		capacity = GetVehicleProperty(v, 0x0D, ShipVehInfo(v->engine_type)->capacity);
-	}
-	_returned_refit_capacity = capacity;
-
-	if (new_cid != v->cargo_type) {
-		cost = GetRefitCost(v->engine_type);
-	}
+	CommandCost cost = RefitVehicle(v, true, new_cid, new_subtype, flags);
 
 	if (flags & DC_EXEC) {
-		v->cargo_cap = capacity;
-		v->cargo.Truncate((v->cargo_type == new_cid) ? capacity : 0);
-		v->cargo_type = new_cid;
-		v->cargo_subtype = new_subtype;
 		v->colourmap = PAL_NONE; // invalidate vehicle colour map
-		v->InvalidateNewGRFCacheOfChain();
-		InvalidateWindow(WC_VEHICLE_DETAILS, v->index);
-		InvalidateWindow(WC_VEHICLE_DEPOT, v->tile);
+		SetWindowDirty(WC_VEHICLE_DETAILS, v->index);
+		SetWindowDirty(WC_VEHICLE_DEPOT, v->tile);
 		InvalidateWindowClassesData(WC_SHIPS_LIST, 0);
 	}
+	v->InvalidateNewGRFCacheOfChain(); // always invalidate; querycost might have filled it
 
 	return cost;
 
